@@ -23,71 +23,77 @@
 
 #include "interfaces.h"
 #include "mpdpp.h"
+#include "regex_filter.h"
 #include "screen.h"
+#include "song_list.h"
 
-struct Browser: Screen<NC::Menu<MPD::Item>>, Filterable, HasSongs, Searchable, Tabbable
+struct BrowserWindow: NC::Menu<MPD::Item>, SongList
+{
+	BrowserWindow() { }
+	BrowserWindow(NC::Menu<MPD::Item> &&base)
+	: NC::Menu<MPD::Item>(std::move(base)) { }
+
+	virtual SongIterator currentS() OVERRIDE;
+	virtual ConstSongIterator currentS() const OVERRIDE;
+	virtual SongIterator beginS() OVERRIDE;
+	virtual ConstSongIterator beginS() const OVERRIDE;
+	virtual SongIterator endS() OVERRIDE;
+	virtual ConstSongIterator endS() const OVERRIDE;
+
+	virtual std::vector<MPD::Song> getSelectedSongs() OVERRIDE;
+};
+
+struct Browser: Screen<BrowserWindow>, HasSongs, Searchable, Tabbable
 {
 	Browser();
 	
-	// Screen< NC::Menu<MPD::Item> > implementation
+	// Screen<BrowserWindow> implementation
 	virtual void resize() OVERRIDE;
 	virtual void switchTo() OVERRIDE;
 	
 	virtual std::wstring title() OVERRIDE;
 	virtual ScreenType type() OVERRIDE { return ScreenType::Browser; }
 	
-	virtual void update() OVERRIDE { }
+	virtual void update() OVERRIDE;
 	
-	virtual void enterPressed() OVERRIDE;
-	virtual void spacePressed() OVERRIDE;
 	virtual void mouseButtonPressed(MEVENT me) OVERRIDE;
 	
+	virtual bool isLockable() OVERRIDE { return true; }
 	virtual bool isMergable() OVERRIDE { return true; }
 	
-	// Filterable implementation
-	virtual bool allowsFiltering() OVERRIDE;
-	virtual std::string currentFilter() OVERRIDE;
-	virtual void applyFilter(const std::string &filter) OVERRIDE;
-	
 	// Searchable implementation
 	virtual bool allowsSearching() OVERRIDE;
-	virtual bool search(const std::string &constraint) OVERRIDE;
-	virtual void nextFound(bool wrap) OVERRIDE;
-	virtual void prevFound(bool wrap) OVERRIDE;
+	virtual void setSearchConstraint(const std::string &constraint) OVERRIDE;
+	virtual void clearConstraint() OVERRIDE;
+	virtual bool find(SearchDirection direction, bool wrap, bool skip_current) OVERRIDE;
 	
 	// HasSongs implementation
-	virtual ProxySongList proxySongList() OVERRIDE;
-	
-	virtual bool allowsSelection() OVERRIDE;
-	virtual void reverseSelection() OVERRIDE;
-	virtual MPD::SongList getSelectedSongs() OVERRIDE;
+	virtual bool itemAvailable() OVERRIDE;
+	virtual bool addItemToPlaylist(bool play) OVERRIDE;
+	virtual std::vector<MPD::Song> getSelectedSongs() OVERRIDE;
 	
 	// private members
-	const std::string &CurrentDir() { return itsBrowsedDir; }
-	
-	void fetchSupportedExtensions();
-	
-	bool isLocal() { return itsBrowseLocally; }
-	void LocateSong(const MPD::Song &);
-	void GetDirectory(std::string, std::string = "/");
-#	ifndef WIN32
-	void GetLocalDirectory(MPD::ItemList &, const std::string &, bool) const;
-	void ClearDirectory(const std::string &) const;
-	void ChangeBrowseMode();
-	bool deleteItem(const MPD::Item &, std::string &errmsg);
-#	endif // !WIN32
-	
-	static bool isParentDirectory(const MPD::Item &item) {
-		return item.type == MPD::itDirectory && item.name == "..";
-	}
-	
-protected:
-	virtual bool isLockable() OVERRIDE { return true; }
+	void requestUpdate() { m_update_request = true; }
+
+	bool inRootDirectory();
+	bool isParentDirectory(const MPD::Item &item);
+	const std::string &currentDirectory();
 	
+	bool isLocal() { return m_local_browser; }
+	void locateSong(const MPD::Song &s);
+	bool enterDirectory();
+	void getDirectory(std::string directory);
+	void changeBrowseMode();
+	void remove(const MPD::Item &item);
+
+	static void fetchSupportedExtensions();
+
 private:
-	bool itsBrowseLocally;
-	size_t itsScrollBeginning;
-	std::string itsBrowsedDir;
+	bool m_update_request;
+	bool m_local_browser;
+	size_t m_scroll_beginning;
+	std::string m_current_directory;
+	Regex::Filter<MPD::Item> m_search_predicate;
 };
 
 extern Browser *myBrowser;
