@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2012 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2014 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,19 +24,22 @@
 
 #ifdef ENABLE_CLOCK
 
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <cstring>
-#include <sys/time.h>
 
 #include "global.h"
 #include "playlist.h"
 #include "settings.h"
 #include "status.h"
+#include "statusbar.h"
+#include "title.h"
+#include "screen_switcher.h"
 
 using Global::MainHeight;
 using Global::MainStartY;
 using Global::myScreen;
 
-Clock *myClock = new Clock;
+Clock *myClock;
 
 short Clock::disp[11] =
 {
@@ -50,74 +53,54 @@ long Clock::older[6], Clock::next[6], Clock::newer[6], Clock::mask;
 size_t Clock::Width;
 const size_t Clock::Height = 8;
 
-void Clock::Init()
+Clock::Clock()
 {
 	Width = Config.clock_display_seconds ? 60 : 40;
 	
-	itsPane = new Window(0, MainStartY, COLS, MainHeight, "", Config.main_color, brNone);
-	w = new Window((COLS-Width)/2, (MainHeight-Height)/2+MainStartY, Width, Height-1, "", Config.main_color, Border(Config.main_color));
-	isInitialized = 1;
+	m_pane = NC::Window(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border());
+	w = NC::Window((COLS-Width)/2, (MainHeight-Height)/2+MainStartY, Width, Height-1, "", Config.main_color, Config.main_color);
 }
 
-void Clock::Resize()
+void Clock::resize()
 {
 	size_t x_offset, width;
-	GetWindowResizeParams(x_offset, width);
+	getWindowResizeParams(x_offset, width);
 	
 	// used for clearing area out of clock window while resizing terminal
-	itsPane->Resize(width, MainHeight);
-	itsPane->MoveTo(x_offset, MainStartY);
-	itsPane->Refresh();
+	m_pane.resize(width, MainHeight);
+	m_pane.moveTo(x_offset, MainStartY);
+	m_pane.refresh();
 	
 	if (Width <= width && Height <= MainHeight)
-		w->MoveTo(x_offset+(width-Width)/2, MainStartY+(MainHeight-Height)/2);
+		w.moveTo(x_offset+(width-Width)/2, MainStartY+(MainHeight-Height)/2);
 }
 
-void Clock::SwitchTo()
+void Clock::switchTo()
 {
-	using Global::myLockedScreen;
-	
-	if (myScreen == this)
-		return;
-	
-	if (!isInitialized)
-		Init();
-	
-	if (myLockedScreen)
-		UpdateInactiveScreen(this);
-	
 	size_t x_offset, width;
-	GetWindowResizeParams(x_offset, width, false);
+	getWindowResizeParams(x_offset, width, false);
 	if (Width > width || Height > MainHeight)
+		Statusbar::print("Screen is too small to display clock");
+	else
 	{
-		ShowMessage("Screen is too small to display clock!");
-		if (myLockedScreen)
-			UpdateInactiveScreen(myLockedScreen);
-		return;
+		SwitchTo::execute(this);
+		drawHeader();
+		Prepare();
+		m_pane.refresh();
+		// clearing screen apparently fixes the problem with last digits being misrendered
+		w.clear();
+		w.display();
 	}
-	
-	if (hasToBeResized || myLockedScreen)
-		Resize();
-	
-	if (myScreen != this && myScreen->isTabbable())
-		Global::myPrevScreen = myScreen;
-	myScreen = this;
-	Global::RedrawHeader = 1;
-	Prepare();
-	itsPane->Refresh();
-	// clearing screen apparently fixes the problem with last digits being misrendered
-	w->Clear();
-	w->Display();
 }
 
-std::basic_string<my_char_t> Clock::Title()
+std::wstring Clock::title()
 {
-	return U("Clock");
+	return L"Clock";
 }
 
-void Clock::Update()
+void Clock::update()
 {
-	if (Width > itsPane->GetWidth() || Height > MainHeight)
+	if (Width > m_pane.getWidth() || Height > MainHeight)
 	{
 		using Global::myLockedScreen;
 		using Global::myInactiveScreen;
@@ -126,32 +109,29 @@ void Clock::Update()
 		{
 			if (myInactiveScreen != myLockedScreen)
 				myScreen = myInactiveScreen;
-			myLockedScreen->SwitchTo();
+			myLockedScreen->switchTo();
 		}
 		else
-			myPlaylist->SwitchTo();
+			myPlaylist->switchTo();
 	}
 	
-	static timeval past = { 0, 0 };
-	gettimeofday(&past, 0);
-	
-	tm *time = localtime(&past.tv_sec);
+	auto time = boost::posix_time::to_tm(Global::Timer);
 	
 	mask = 0;
-	Set(time->tm_sec % 10, 0);
-	Set(time->tm_sec / 10, 4);
-	Set(time->tm_min % 10, 10);
-	Set(time->tm_min / 10, 14);
-	Set(time->tm_hour % 10, 20);
-	Set(time->tm_hour / 10, 24);
+	Set(time.tm_sec % 10, 0);
+	Set(time.tm_sec / 10, 4);
+	Set(time.tm_min % 10, 10);
+	Set(time.tm_min / 10, 14);
+	Set(time.tm_hour % 10, 20);
+	Set(time.tm_hour / 10, 24);
 	Set(10, 7);
 	Set(10, 17);
 	
 	char buf[64];
-	strftime(buf, 64, "%x", time);
-	attron(COLOR_PAIR(Config.main_color));
-	mvprintw(w->GetStartY()+w->GetHeight(), w->GetStartX()+(w->GetWidth()-strlen(buf))/2, "%s", buf);
-	attroff(COLOR_PAIR(Config.main_color));
+	std::strftime(buf, 64, "%x", &time);
+	color_set(Config.main_color.pairNumber(), nullptr);
+	mvprintw(w.getStarty()+w.getHeight(), w.getStartX()+(w.getWidth()-strlen(buf))/2, "%s", buf);
+	standend();
 	refresh();
 	
 	for (int k = 0; k < 6; ++k)
@@ -160,7 +140,7 @@ void Clock::Update()
 		next[k] = 0;
 		for (int s = 1; s >= 0; --s)
 		{
-			*w << (s ? fmtReverse : fmtReverseEnd);
+			w << (s ? NC::Format::Reverse : NC::Format::NoReverse);
 			for (int i = 0; i < 6; ++i)
 			{
 				long a = (newer[i] ^ older[i]) & (s ? newer : older)[i];
@@ -173,10 +153,10 @@ void Clock::Update()
 						{
 							if (!(a & (t << 1)))
 							{
-								w->GotoXY(2*j+2, i);
+								w.goToXY(2*j+2, i);
 							}
 							if (Config.clock_display_seconds || j < 18)
-								*w << "  ";
+								w << "  ";
 						}
 					}
 				}
@@ -187,7 +167,7 @@ void Clock::Update()
 			}
 		}
 	}
-	w->Refresh();
+	w.refresh();
 }
 
 void Clock::Prepare()
