@@ -25,6 +25,7 @@
 #include "help.h"
 #include "settings.h"
 #include "status.h"
+#include "utility/string.h"
 #include "utility/wide_string.h"
 #include "title.h"
 #include "screen_switcher.h"
@@ -36,90 +37,48 @@ Help *myHelp;
 
 namespace {
 
-std::string key_to_string(const Key &key, bool *print_backspace)
+std::string align_key_rep(std::wstring keys)
 {
-	std::string result;
-	if (key == Key(KEY_UP, Key::NCurses))
-		result += "Up";
-	else if (key == Key(KEY_DOWN, Key::NCurses))
-		result += "Down";
-	else if (key == Key(KEY_PPAGE, Key::NCurses))
-		result += "Page Up";
-	else if (key == Key(KEY_NPAGE, Key::NCurses))
-		result += "Page Down";
-	else if (key == Key(KEY_HOME, Key::NCurses))
-		result += "Home";
-	else if (key == Key(KEY_END, Key::NCurses))
-		result += "End";
-	else if (key == Key(KEY_SPACE, Key::Standard))
-		result += "Space";
-	else if (key == Key(KEY_ENTER, Key::Standard))
-		result += "Enter";
-	else if (key == Key(KEY_IC, Key::NCurses))
-		result += "Insert";
-	else if (key == Key(KEY_DC, Key::NCurses))
-		result += "Delete";
-	else if (key == Key(KEY_RIGHT, Key::NCurses))
-		result += "Right";
-	else if (key == Key(KEY_LEFT, Key::NCurses))
-		result += "Left";
-	else if (key == Key(KEY_TAB, Key::Standard))
-		result += "Tab";
-	else if (key == Key(KEY_SHIFT_TAB, Key::NCurses))
-		result += "Shift-Tab";
-	else if (key >= Key(KEY_CTRL_A, Key::Standard) && key <= Key(KEY_CTRL_Z, Key::Standard))
+	size_t i = 0, len = 0;
+	const size_t max_len = 20;
+	for (; i < keys.size(); ++i)
 	{
-		result += "Ctrl-";
-		result += key.getChar()+64;
+		int width = std::max(1, wcwidth(keys[i]));
+		if (len+width > max_len)
+			break;
+		else
+			len += width;
 	}
-	else if (key >= Key(KEY_F1, Key::NCurses) && key <= Key(KEY_F12, Key::NCurses))
-	{
-		result += "F";
-		result += boost::lexical_cast<std::string>(key.getChar()-264);
-	}
-	else if ((key == Key(KEY_BACKSPACE, Key::NCurses) || key == Key(KEY_BACKSPACE_2, Key::Standard)))
-	{
-		// since some terminals interpret KEY_BACKSPACE as backspace and other need KEY_BACKSPACE_2,
-		// actions have to be bound to either of them, but we want to display "Backspace" only once,
-		// hance this 'print_backspace' switch.
-		if (!print_backspace || *print_backspace)
-		{
-			result += "Backspace";
-			if (print_backspace)
-				*print_backspace = false;
-		}
-	}
-	else
-		result += ToString(std::wstring(1, key.getChar()));
-	return result;
+	keys.resize(i + max_len - len, ' ');
+	return ToString(keys);
 }
 
 std::string display_keys(const Actions::Type at)
 {
-	bool print_backspace = true;
-	std::string result, skey;
+	std::wstring result, skey;
 	for (auto it = Bindings.begin(); it != Bindings.end(); ++it)
 	{
 		for (auto j = it->second.begin(); j != it->second.end(); ++j)
 		{
 			if (j->isSingle() && j->action()->type() == at)
 			{
-				skey = key_to_string(it->first, &print_backspace);
+				skey = keyToWString(it->first);
 				if (!skey.empty())
 				{
 					result += std::move(skey);
-					result += " ";
+					result += ' ';
 				}
 			}
 		}
 	}
-	result.resize(16, ' ');
-	return result;
+	return align_key_rep(std::move(result));
 }
 
 void section(NC::Scrollpad &w, const char *type_, const char *title_)
 {
-	w << "\n  " << NC::Format::Bold << type_ << " - ";
+	w << "\n  " << NC::Format::Bold;
+	if (type_[0] != '\0')
+		w << type_ << " - ";
 	w << title_ << NC::Format::NoBold << "\n\n";
 }
 
@@ -140,6 +99,11 @@ void key(NC::Scrollpad &w, const Actions::Type at, const boost::format &desc)
 	w << "    " << display_keys(at) << " : " << desc.str() << '\n';
 }
 
+void key(NC::Scrollpad &w, NC::Key::Type k, const std::string &desc)
+{
+	w << "    " << align_key_rep(keyToWString(k)) << " : " << desc << '\n';
+}
+
 /**********************************************************************/
 
 void mouse_section(NC::Scrollpad &w, const char *title_)
@@ -223,11 +187,13 @@ void write_bindings(NC::Scrollpad &w)
 		boost::format("Increase volume by %1%%%") % Config.volume_change_step
 	);
 	w << '\n';
-	key(w, Type::ToggleSpaceMode, "Toggle space mode (select/add)");
 	key(w, Type::ToggleAddMode, "Toggle add mode (add or remove/always add)");
 	key(w, Type::ToggleMouse, "Toggle mouse support");
+	key(w, Type::SelectRange, "Select range");
 	key(w, Type::ReverseSelection, "Reverse selection");
 	key(w, Type::RemoveSelection, "Remove selection");
+	key(w, Type::SelectItem, "Select current item");
+	key(w, Type::SelectFoundItems, "Select found items");
 	key(w, Type::SelectAlbum, "Select songs of album around the cursor");
 	key(w, Type::AddSelectedItems, "Add selected items to playlist");
 	key(w, Type::AddRandomItems, "Add random items to playlist");
@@ -238,14 +204,12 @@ void write_bindings(NC::Scrollpad &w)
 	key(w, Type::ToggleConsume, "Toggle consume mode");
 	key(w, Type::ToggleReplayGainMode, "Toggle replay gain mode");
 	key(w, Type::ToggleBitrateVisibility, "Toggle bitrate visibility");
-	key(w, Type::Shuffle, "Shuffle playlist");
 	key(w, Type::ToggleCrossfade, "Toggle crossfade mode");
 	key(w, Type::SetCrossfade, "Set crossfade");
 	key(w, Type::SetVolume, "Set volume");
 	key(w, Type::UpdateDatabase, "Start music database update");
 	w << '\n';
 	key(w, Type::ExecuteCommand, "Execute command");
-	key(w, Type::ApplyFilter, "Apply filter");
 	key(w, Type::FindItemForward, "Find item forward");
 	key(w, Type::FindItemBackward, "Find item backward");
 	key(w, Type::PreviousFoundItem, "Jump to previous found item");
@@ -274,7 +238,7 @@ void write_bindings(NC::Scrollpad &w)
 	key(w, Type::Quit, "Quit");
 
 	key_section(w, "Playlist");
-	key(w, Type::PressEnter, "Play selected item");
+	key(w, Type::PlayItem, "Play selected item");
 	key(w, Type::DeletePlaylistItems, "Delete selected item(s) from playlist");
 	key(w, Type::ClearMainPlaylist, "Clear playlist");
 	key(w, Type::CropMainPlaylist, "Clear playlist except selected item(s)");
@@ -287,15 +251,16 @@ void write_bindings(NC::Scrollpad &w)
 	key(w, Type::EditSong, "Edit song");
 #	endif // HAVE_TAGLIB_H
 	key(w, Type::SavePlaylist, "Save playlist");
-	key(w, Type::SortPlaylist, "Sort playlist");
-	key(w, Type::ReversePlaylist, "Reverse playlist");
-	key(w, Type::FilterPlaylistOnPriorities, "Filter playlist on priorities");
+	key(w, Type::Shuffle, "Shuffle range");
+	key(w, Type::SortPlaylist, "Sort range");
+	key(w, Type::ReversePlaylist, "Reverse range");
 	key(w, Type::JumpToPlayingSong, "Jump to current song");
 	key(w, Type::TogglePlayingSongCentering, "Toggle playing song centering");
 
 	key_section(w, "Browser");
-	key(w, Type::PressEnter, "Enter directory/Add item to playlist and play it");
-	key(w, Type::PressSpace, "Add item to playlist/select it");
+	key(w, Type::EnterDirectory, "Enter directory");
+	key(w, Type::PlayItem, "Add item to playlist and play it");
+	key(w, Type::AddItemToPlaylist, "Add item to playlist");
 #	ifdef HAVE_TAGLIB_H
 	key(w, Type::EditSong, "Edit song");
 #	endif // HAVE_TAGLIB_H
@@ -309,8 +274,9 @@ void write_bindings(NC::Scrollpad &w)
 	key(w, Type::JumpToPlaylistEditor, "Jump to playlist editor (playlists only)");
 
 	key_section(w, "Search engine");
-	key(w, Type::PressEnter, "Add item to playlist and play it/change option");
-	key(w, Type::PressSpace, "Add item to playlist");
+	key(w, Type::RunAction, "Modify option / Run action");
+	key(w, Type::AddItemToPlaylist, "Add item to playlist");
+	key(w, Type::PlayItem, "Add item to playlist and play it");
 #	ifdef HAVE_TAGLIB_H
 	key(w, Type::EditSong, "Edit song");
 #	endif // HAVE_TAGLIB_H
@@ -321,8 +287,8 @@ void write_bindings(NC::Scrollpad &w)
 	key(w, Type::ToggleMediaLibraryColumnsMode, "Switch between two/three columns mode");
 	key(w, Type::PreviousColumn, "Previous column");
 	key(w, Type::NextColumn, "Next column");
-	key(w, Type::PressEnter, "Add item to playlist and play it");
-	key(w, Type::PressSpace, "Add item to playlist");
+	key(w, Type::PlayItem, "Add item to playlist and play it");
+	key(w, Type::AddItemToPlaylist, "Add item to playlist");
 #	ifdef HAVE_TAGLIB_H
 	key(w, Type::EditSong, "Edit song");
 #	endif // HAVE_TAGLIB_H
@@ -333,8 +299,8 @@ void write_bindings(NC::Scrollpad &w)
 	key_section(w, "Playlist editor");
 	key(w, Type::PreviousColumn, "Previous column");
 	key(w, Type::NextColumn, "Next column");
-	key(w, Type::PressEnter, "Add item to playlist and play it");
-	key(w, Type::PressSpace, "Add item to playlist/select it");
+	key(w, Type::PlayItem, "Add item to playlist and play it");
+	key(w, Type::AddItemToPlaylist, "Add item to playlist");
 #	ifdef HAVE_TAGLIB_H
 	key(w, Type::EditSong, "Edit song");
 #	endif // HAVE_TAGLIB_H
@@ -347,20 +313,19 @@ void write_bindings(NC::Scrollpad &w)
 	key(w, Type::CropPlaylist, "Clear playlist except selected items");
 
 	key_section(w, "Lyrics");
-	key(w, Type::PressSpace, "Toggle reloading lyrics upon song change");
+	key(w, Type::ToggleLyricsUpdateOnSongChange, "Toggle lyrics update on song change");
 	key(w, Type::EditLyrics, "Open lyrics in external editor");
 	key(w, Type::RefetchLyrics, "Refetch lyrics");
 
 #	ifdef HAVE_TAGLIB_H
 	key_section(w, "Tiny tag editor");
-	key(w, Type::PressEnter, "Edit tag");
+	key(w, Type::RunAction, "Edit tag / Run action");
 	key(w, Type::SaveTagChanges, "Save");
 
 	key_section(w, "Tag editor");
-	key(w, Type::PressEnter, "Edit tag/filename of selected item (left column)");
-	key(w, Type::PressEnter, "Perform operation on all/selected items (middle column)");
-	key(w, Type::PressSpace, "Switch to albums/directories view (left column)");
-	key(w, Type::PressSpace, "Select item (right column)");
+	key(w, Type::EnterDirectory, "Enter directory (right column)");
+	key(w, Type::RunAction, "Perform operation on selected items (middle column)");
+	key(w, Type::RunAction, "Edit item (left column)");
 	key(w, Type::PreviousColumn, "Previous column");
 	key(w, Type::NextColumn, "Next column");
 	key(w, Type::JumpToParentDirectory, "Jump to parent directory (left column, directories view)");
@@ -368,12 +333,12 @@ void write_bindings(NC::Scrollpad &w)
 
 #	ifdef ENABLE_OUTPUTS
 	key_section(w, "Outputs");
-	key(w, Type::PressEnter, "Toggle output");
+	key(w, Type::ToggleOutput, "Toggle output");
 #	endif // ENABLE_OUTPUTS
 
 #	if defined(ENABLE_VISUALIZER) && defined(HAVE_FFTW3_H)
 	key_section(w, "Music visualizer");
-	key(w, Type::PressSpace, "Toggle visualization type");
+	key(w, Type::ToggleVisualizationType, "Toggle visualization type");
 	key(w, Type::SetVisualizerSampleMultiplier, "Set visualizer sample multiplier");
 #	endif // ENABLE_VISUALIZER && HAVE_FFTW3_H
 
@@ -381,7 +346,7 @@ void write_bindings(NC::Scrollpad &w)
 	mouse(w, "Left click on \"Playing/Paused\"", "Play/pause");
 	mouse(w, "Left click on progressbar", "Jump to pointed position in playing song");
 	w << '\n';
-	mouse(w, "Mouse wheel on \"Volume: xx\"", "Play/pause");
+	mouse(w, "Mouse wheel on \"Volume: xx\"", "Adjust volume");
 	mouse(w, "Mouse wheel on main window", "Scroll");
 
 	mouse_section(w, "Playlist");
@@ -442,12 +407,30 @@ void write_bindings(NC::Scrollpad &w)
 	mouse(w, "Right click", "Toggle output");
 #	endif // ENABLE_OUTPUTS
 
+	section(w, "", "Action chains");
+	for (const auto &k : Bindings)
+	{
+		for (const auto &binding : k.second)
+		{
+			if (!binding.isSingle())
+			{
+				std::vector<std::string> commands;
+				for (const auto &action : binding.actions())
+					commands.push_back(action->name());
+				key(w, k.first, join<std::string>(commands, ", "));
+			}
+		}
+	}
+
+	section(w, "", "List of available colors");
+	for (int i = 0; i < COLORS; ++i)
+		w << NC::Color(i, NC::Color::transparent) << i+1 << NC::Color::End << " ";
 }
 
 }
 
 Help::Help()
-: Screen(NC::Scrollpad(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border::None))
+: Screen(NC::Scrollpad(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border()))
 {
 	write_bindings(w);
 	w.flush();
