@@ -37,13 +37,8 @@ struct Help: Screen<NC::Scrollpad>, Tabbable
 	
 	virtual void update() OVERRIDE { }
 	
-	virtual void enterPressed() OVERRIDE { }
-	virtual void spacePressed() OVERRIDE { }
-	
-	virtual bool isMergable() OVERRIDE { return true; }
-	
-protected:
 	virtual bool isLockable() OVERRIDE { return true; }
+	virtual bool isMergable() OVERRIDE { return true; }
 };
 
 extern Help *myHelp;
