@@ -19,68 +19,168 @@
  ***************************************************************************/
 
 #include <algorithm>
+#include <time.h>
 
+#include "enums.h"
 #include "helpers.h"
 #include "playlist.h"
 #include "statusbar.h"
+#include "utility/functional.h"
+
+const MPD::Song *currentSong(const BaseScreen *screen)
+{
+	const MPD::Song *ptr = nullptr;
+	const auto *list = dynamic_cast<const SongList *>(screen->activeWindow());
+	if (list != nullptr)
+	{
+		const auto it = list->currentS();
+		if (it != list->endS())
+			ptr = it->get<Bit::Song>();
+	}
+	return ptr;
+}
+
+MPD::SongIterator getDatabaseIterator(MPD::Connection &mpd)
+{
+	MPD::SongIterator result;
+	try
+	{
+		result = mpd.GetDirectoryRecursive("/");
+	}
+	catch (MPD::ClientError &e)
+	{
+		if (e.code() == MPD_ERROR_CLOSED)
+		{
+			// If we can't get the database, display appropriate
+			// error message and reconnect with the MPD server.
+			Statusbar::print("Unable to fetch the data, increase max_buffer_output_size in your MPD configuration file");
+			mpd.Disconnect();
+			mpd.Connect();
+		}
+		else
+			throw;
+	}
+	return result;
+}
+
+typedef std::vector<MPD::Song>::const_iterator VectorSongIterator;
+bool addSongsToPlaylist(VectorSongIterator first, VectorSongIterator last, bool play, int position)
+{
+	bool result = true;
+	auto addSongNoError = [&](VectorSongIterator song) -> int {
+		try
+		{
+			return Mpd.AddSong(*song, position);
+		}
+		catch (MPD::ServerError &e)
+		{
+			Status::handleServerError(e);
+			result = false;
+			return -1;
+		}
+	};
+
+	if (last-first >= 1)
+	{
+		int id;
+		while (true)
+		{
+			id = addSongNoError(first);
+			if (id >= 0)
+				break;
+			++first;
+			if (first == last)
+				return result;
+		}
+
+		if (position == -1)
+		{
+			++first;
+			for(; first != last; ++first)
+				addSongNoError(first);
+		}
+		else
+		{
+			++position;
+			--last;
+			for (; first != last; --last)
+				addSongNoError(last);
+		}
+		if (play)
+			Mpd.PlayID(id);
+	}
+
+	return result;
+}
+
+void removeSongFromPlaylist(const SongMenu &playlist, const MPD::Song &s)
+{
+	Mpd.StartCommandsList();
+	for (auto &item : reverse_iteration(playlist))
+		if (item.value() == s)
+			Mpd.Delete(item.value().getPosition());
+	Mpd.CommitCommandsList();
+}
 
 bool addSongToPlaylist(const MPD::Song &s, bool play, int position)
 {
 	bool result = false;
-	if (Config.space_add_mode == SpaceAddMode::AddRemove && myPlaylist->checkForSong(s))
+	if (Config.space_add_mode == SpaceAddMode::AddRemove
+	&&  myPlaylist->checkForSong(s)
+	   )
 	{
-		auto &w = myPlaylist->main();
+		result = true;
 		if (play)
 		{
-			auto song = std::find(w.beginV(), w.endV(), s);
-			assert(song != w.endV());
-			Mpd.PlayID(song->getID());
-			result = true;
+			const auto begin = myPlaylist->main().beginV(), end = myPlaylist->main().endV();
+			auto it = find_map_first(begin, end, s, [](const MPD::Song &found) {
+				Mpd.PlayID(found.getID());
+			});
+			assert(it != end);
 		}
 		else
-		{
-			Mpd.StartCommandsList();
-			for (auto it = w.rbeginV(); it != w.rendV(); ++it)
-				if (*it == s)
-					Mpd.Delete(it->getPosition());
-			Mpd.CommitCommandsList();
-			// we return false in this case
-		}
+			removeSongFromPlaylist(myPlaylist->main(), s);
+		return result;
 	}
-	else
+	int id = Mpd.AddSong(s, position);
+	if (id >= 0)
 	{
-		int id = Mpd.AddSong(s, position);
-		if (id >= 0)
-		{
-			Statusbar::printf("Added to playlist: %s",
-				s.toString(Config.song_status_format_no_colors, Config.tags_separator)
-			);
-			if (play)
-				Mpd.PlayID(id);
-			result = true;
-		}
+		Statusbar::printf("Added to playlist: %s",
+			Format::stringify<char>(Config.song_status_format, &s)
+		);
+		if (play)
+			Mpd.PlayID(id);
+		result = true;
 	}
 	return result;
 }
 
+std::string timeFormat(const char *format, time_t t)
+{
+	char result[32];
+	tm tinfo;
+	localtime_r(&t, &tinfo);
+	strftime(result, sizeof(result), format, &tinfo);
+	return result;
+}
+
 std::string Timestamp(time_t t)
 {
 	char result[32];
-#	ifdef WIN32
-	result[strftime(result, 31, "%x %X", localtime(&t))] = 0;
-#	else
 	tm info;
 	result[strftime(result, 31, "%x %X", localtime_r(&t, &info))] = 0;
-#	endif // WIN32
 	return result;
 }
 
-void markSongsInPlaylist(ProxySongList pl)
+void markSongsInPlaylist(SongList &list)
 {
-	size_t list_size = pl.size();
-	for (size_t i = 0; i < list_size; ++i)
-		if (auto s = pl.getSong(i))
-			pl.setBold(i, myPlaylist->checkForSong(*s));
+	MPD::Song *s;
+	for (auto &p : list)
+	{
+		s = p.get<Bit::Song>();
+		if (s != nullptr)
+			p.get<Bit::Properties>().setBold(myPlaylist->checkForSong(*s));
+	}
 }
 
 std::wstring Scroller(const std::wstring &str, size_t &pos, size_t width)
@@ -128,21 +228,21 @@ void writeCyclicBuffer(const NC::WBuffer &buf, NC::Window &w, size_t &start_pos,
 		auto p = ps.begin();
 		
 		// load attributes from before starting pos
-		for (; p != ps.end() && p->position() < start_pos; ++p)
-			w << *p;
+		for (; p != ps.end() && p->first < start_pos; ++p)
+			w << p->second;
 		
 		auto write_buffer = [&](size_t start) {
 			for (size_t i = start; i < s.length() && len < width; ++i)
 			{
-				for (; p != ps.end() && p->position() == i; ++p)
-					w << *p;
+				for (; p != ps.end() && p->first == i; ++p)
+					w << p->second;
 				len += wcwidth(s[i]);
 				if (len > width)
 					break;
 				w << s[i];
 			}
 			for (; p != ps.end(); ++p)
-				w << *p;
+				w << p->second;
 			p = ps.begin();
 		};
 		
