@@ -35,7 +35,7 @@ using Global::MainStartY;
 Lastfm *myLastfm;
 
 Lastfm::Lastfm()
-: Screen(NC::Scrollpad(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border::None))
+: Screen(NC::Scrollpad(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border()))
 { }
 
 void Lastfm::resize()
