@@ -18,7 +18,6 @@
  *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
-#include <boost/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/locale/conversion.hpp>
 #include <algorithm>
@@ -29,13 +28,15 @@
 #include "display.h"
 #include "helpers.h"
 #include "global.h"
-#include "media_library.h"
+#include "menu_impl.h"
 #include "mpdpp.h"
 #include "playlist.h"
-#include "regex_filter.h"
+#include "media_library.h"
 #include "status.h"
 #include "statusbar.h"
+#include "helpers/song_iterator_maker.h"
 #include "utility/comparators.h"
+#include "utility/functional.h"
 #include "utility/type_conversions.h"
 #include "title.h"
 #include "screen_switcher.h"
@@ -44,6 +45,8 @@ using Global::MainHeight;
 using Global::MainStartY;
 using Global::myScreen;
 
+namespace ph = std::placeholders;
+
 MediaLibrary *myLibrary;
 
 namespace {
@@ -56,10 +59,10 @@ size_t itsMiddleColStartX;
 size_t itsRightColWidth;
 size_t itsRightColStartX;
 
+typedef MediaLibrary::PrimaryTag PrimaryTag;
 typedef MediaLibrary::AlbumEntry AlbumEntry;
 
-template <typename FunT>
-void withSongsFromAlbum(const AlbumEntry &album, FunT &&f)
+MPD::SongIterator getSongsFromAlbum(const AlbumEntry &album)
 {
 	Mpd.StartSearch(true);
 	Mpd.AddSearch(Config.media_lib_primary_tag, album.entry().tag());
@@ -68,15 +71,18 @@ void withSongsFromAlbum(const AlbumEntry &album, FunT &&f)
 		Mpd.AddSearch(MPD_TAG_ALBUM, album.entry().album());
 		Mpd.AddSearch(MPD_TAG_DATE, album.entry().date());
 	}
-	Mpd.CommitSearchSongs(std::forward<FunT>(f));
+	return Mpd.CommitSearchSongs();
 }
 
 std::string AlbumToString(const AlbumEntry &ae);
 std::string SongToString(const MPD::Song &s);
 
-bool TagEntryMatcher(const boost::regex &rx, const MediaLibrary::PrimaryTag &tagmtime);
-bool AlbumEntryMatcher(const boost::regex &rx, const NC::Menu<AlbumEntry>::Item &item, bool filter);
-bool SongEntryMatcher(const boost::regex &rx, const MPD::Song &s);
+bool TagEntryMatcher(const Regex::Regex &rx, const MediaLibrary::PrimaryTag &tagmtime);
+bool AlbumEntryMatcher(const Regex::Regex &rx, const NC::Menu<AlbumEntry>::Item &item, bool filter);
+bool SongEntryMatcher(const Regex::Regex &rx, const MPD::Song &s);
+
+bool MoveToTag(NC::Menu<PrimaryTag> &tags, const std::string &primary_tag);
+bool MoveToAlbum(NC::Menu<AlbumEntry> &albums, const std::string &primary_tag, const MPD::Song &s);
 
 struct SortSongs {
 	typedef NC::Menu<MPD::Song>::Item SongItem;
@@ -95,12 +101,18 @@ public:
 	}
 	bool operator()(const MPD::Song &a, const MPD::Song &b) {
 		for (auto get = GetFuns.begin()+m_offset; get != GetFuns.end(); ++get) {
-			int ret = m_cmp(a.getTags(*get, Config.tags_separator),
-			                b.getTags(*get, Config.tags_separator));
+			int ret = m_cmp(a.getTags(*get),
+			                b.getTags(*get));
 			if (ret != 0)
 				return ret < 0;
 		}
-		return a.getTrack() < b.getTrack();
+		try {
+			int ret = boost::lexical_cast<int>(a.getTags(&MPD::Song::getTrackNumber))
+			        - boost::lexical_cast<int>(b.getTags(&MPD::Song::getTrackNumber));
+			return ret < 0;
+		} catch (boost::bad_lexical_cast &) {
+			return a.getTrackNumber() < b.getTrackNumber();
+		}
 	}
 };
 
@@ -140,8 +152,6 @@ public:
 };
 
 class SortPrimaryTags {
-	typedef MediaLibrary::PrimaryTag PrimaryTag;
-	
 	LocaleStringComparison m_cmp;
 	
 public:
@@ -159,7 +169,7 @@ public:
 
 MediaLibrary::MediaLibrary()
 : m_timer(boost::posix_time::from_time_t(0))
-, m_window_timeout(Config.data_fetching_delay ? 250 : 500)
+, m_window_timeout(Config.data_fetching_delay ? 250 : BaseScreen::defaultWindowTimeout)
 , m_fetching_delay(boost::posix_time::milliseconds(Config.data_fetching_delay ? 250 : -1))
 {
 	hasTwoColumns = 0;
@@ -169,7 +179,7 @@ MediaLibrary::MediaLibrary()
 	itsRightColWidth = COLS-COLS/3*2-1;
 	itsRightColStartX = itsLeftColWidth+itsMiddleColWidth+2;
 	
-	Tags = NC::Menu<PrimaryTag>(0, MainStartY, itsLeftColWidth, MainHeight, Config.titles_visibility ? tagTypeToString(Config.media_lib_primary_tag) + "s" : "", Config.main_color, NC::Border::None);
+	Tags = NC::Menu<PrimaryTag>(0, MainStartY, itsLeftColWidth, MainHeight, Config.titles_visibility ? tagTypeToString(Config.media_lib_primary_tag) + "s" : "", Config.main_color, NC::Border());
 	Tags.setHighlightColor(Config.active_column_color);
 	Tags.cyclicScrolling(Config.use_cyclic_scrolling);
 	Tags.centeredCursor(Config.centered_cursor);
@@ -183,7 +193,7 @@ MediaLibrary::MediaLibrary()
 			menu << Charset::utf8ToLocale(tag);
 	});
 	
-	Albums = NC::Menu<AlbumEntry>(itsMiddleColStartX, MainStartY, itsMiddleColWidth, MainHeight, Config.titles_visibility ? "Albums" : "", Config.main_color, NC::Border::None);
+	Albums = NC::Menu<AlbumEntry>(itsMiddleColStartX, MainStartY, itsMiddleColWidth, MainHeight, Config.titles_visibility ? "Albums" : "", Config.main_color, NC::Border());
 	Albums.setHighlightColor(Config.main_highlight_color);
 	Albums.cyclicScrolling(Config.use_cyclic_scrolling);
 	Albums.centeredCursor(Config.centered_cursor);
@@ -193,13 +203,15 @@ MediaLibrary::MediaLibrary()
 		menu << Charset::utf8ToLocale(AlbumToString(menu.drawn()->value()));
 	});
 	
-	Songs = NC::Menu<MPD::Song>(itsRightColStartX, MainStartY, itsRightColWidth, MainHeight, Config.titles_visibility ? "Songs" : "", Config.main_color, NC::Border::None);
+	Songs = NC::Menu<MPD::Song>(itsRightColStartX, MainStartY, itsRightColWidth, MainHeight, Config.titles_visibility ? "Songs" : "", Config.main_color, NC::Border());
 	Songs.setHighlightColor(Config.main_highlight_color);
 	Songs.cyclicScrolling(Config.use_cyclic_scrolling);
 	Songs.centeredCursor(Config.centered_cursor);
 	Songs.setSelectedPrefix(Config.selected_item_prefix);
 	Songs.setSelectedSuffix(Config.selected_item_suffix);
-	Songs.setItemDisplayer(boost::bind(Display::Songs, _1, songsProxyList(), Config.song_library_format));
+	Songs.setItemDisplayer(std::bind(
+		Display::Songs, ph::_1, std::cref(Songs), std::cref(Config.song_library_format)
+	));
 	
 	w = &Tags;
 }
@@ -253,7 +265,7 @@ void MediaLibrary::refresh()
 void MediaLibrary::switchTo()
 {
 	SwitchTo::execute(this);
-	markSongsInPlaylist(songsProxyList());
+	markSongsInPlaylist(Songs);
 	drawHeader();
 	refresh();
 }
@@ -267,147 +279,143 @@ void MediaLibrary::update()
 {
 	if (hasTwoColumns)
 	{
-		if (Albums.reallyEmpty() || m_albums_update_request)
+		if (Albums.empty() || m_albums_update_request)
 		{
-			Albums.clearSearchResults();
 			m_albums_update_request = false;
 			std::map<std::tuple<std::string, std::string, std::string>, time_t> albums;
-			Mpd.GetDirectoryRecursive("/", [&albums](MPD::Song s) {
+			for (MPD::SongIterator s = getDatabaseIterator(Mpd), end; s != end; ++s)
+			{
+				std::string tag;
 				unsigned idx = 0;
-				std::string tag = s.get(Config.media_lib_primary_tag, idx);
-				do
+				while (!(tag = s->get(Config.media_lib_primary_tag, idx++)).empty())
 				{
-					auto key = std::make_tuple(tag, s.getAlbum(), s.getDate());
+					auto key = std::make_tuple(std::move(tag), s->getAlbum(), s->getDate());
 					auto it = albums.find(key);
 					if (it == albums.end())
-						albums[key] = s.getMTime();
+						albums[std::move(key)] = s->getMTime();
 					else
-						it->second = s.getMTime();
-				}
-				while (!(tag = s.get(Config.media_lib_primary_tag, ++idx)).empty());
-			});
-			withUnfilteredMenuReapplyFilter(Albums, [this, &albums]() {
-				size_t idx = 0;
-				for (auto it = albums.begin(); it != albums.end(); ++it, ++idx)
-				{
-					auto &&entry = AlbumEntry(Album(
-						std::move(std::get<0>(it->first)),
-						std::move(std::get<1>(it->first)),
-						std::move(std::get<2>(it->first)),
-						it->second));
-					if (idx < Albums.size())
-						Albums[idx].value() = entry;
-					else
-						Albums.addItem(entry);
+						it->second = s->getMTime();
 				}
+			}
+			size_t idx = 0;
+			for (const auto &album : albums)
+			{
+				auto entry = AlbumEntry(Album(
+					std::move(std::get<0>(album.first)),
+					std::move(std::get<1>(album.first)),
+					std::move(std::get<2>(album.first)),
+					album.second)
+				);
 				if (idx < Albums.size())
-					Albums.resizeList(idx);
-				std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
-			});
+					Albums[idx].value() = std::move(entry);
+				else
+					Albums.addItem(std::move(entry));
+				++idx;
+			}
+			if (idx < Albums.size())
+				Albums.resizeList(idx);
+			std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
 			Albums.refresh();
 		}
 	}
 	else
 	{
-		if (Tags.reallyEmpty() || m_tags_update_request)
+		if (Tags.empty() || m_tags_update_request)
 		{
-			Tags.clearSearchResults();
 			m_tags_update_request = false;
 			std::map<std::string, time_t> tags;
 			if (Config.media_library_sort_by_mtime)
 			{
-				Mpd.GetDirectoryRecursive("/", [&tags](MPD::Song s) {
+				for (MPD::SongIterator s = getDatabaseIterator(Mpd), end; s != end; ++s)
+				{
+					std::string tag;
 					unsigned idx = 0;
-					std::string tag = s.get(Config.media_lib_primary_tag, idx);
-					do
+					while (!(tag = s->get(Config.media_lib_primary_tag, idx++)).empty())
 					{
 						auto it = tags.find(tag);
 						if (it == tags.end())
-							tags[tag] = s.getMTime();
+							tags[std::move(tag)] = s->getMTime();
 						else
-							it->second = std::max(it->second, s.getMTime());
+							it->second = std::max(it->second, s->getMTime());
 					}
-					while (!(tag = s.get(Config.media_lib_primary_tag, ++idx)).empty());
-				});
+				}
 			}
 			else
 			{
-				Mpd.GetList(Config.media_lib_primary_tag, [&tags](std::string tag) {
-					tags[tag] = 0;
-				});
+				MPD::StringIterator tag = Mpd.GetList(Config.media_lib_primary_tag), end;
+				for (; tag != end; ++tag)
+					tags[std::move(*tag)] = 0;
 			}
-			withUnfilteredMenuReapplyFilter(Tags, [this, &tags]() {
-				size_t idx = 0;
-				for (auto it = tags.begin(); it != tags.end(); ++it, ++idx)
-				{
-					auto &&tag = PrimaryTag(std::move(it->first), it->second);
-					if (idx < Tags.size())
-						Tags[idx].value() = tag;
-					else
-						Tags.addItem(tag);
-				}
+			size_t idx = 0;
+			for (const auto &tag : tags)
+			{
+				auto ptag = PrimaryTag(std::move(tag.first), tag.second);
 				if (idx < Tags.size())
-					Tags.resizeList(idx);
-				std::sort(Tags.beginV(), Tags.endV(), SortPrimaryTags());
-			});
+					Tags[idx].value() = std::move(ptag);
+				else
+					Tags.addItem(std::move(ptag));
+				++idx;
+			}
+			if (idx < Tags.size())
+				Tags.resizeList(idx);
+			std::sort(Tags.beginV(), Tags.endV(), SortPrimaryTags());
 			Tags.refresh();
 		}
 		
 		if (!Tags.empty()
-		&& ((Albums.reallyEmpty() && Global::Timer - m_timer > m_fetching_delay) || m_albums_update_request)
+		&& ((Albums.empty() && Global::Timer - m_timer > m_fetching_delay) || m_albums_update_request)
 		)
 		{
-			Albums.clearSearchResults();
 			m_albums_update_request = false;
-			auto &primary_tag = Tags.current().value().tag();
+			auto &primary_tag = Tags.current()->value().tag();
 			Mpd.StartSearch(true);
 			Mpd.AddSearch(Config.media_lib_primary_tag, primary_tag);
 			std::map<std::tuple<std::string, std::string>, time_t> albums;
-			Mpd.CommitSearchSongs([&albums](MPD::Song s) {
-				auto key = std::make_tuple(s.getAlbum(), s.getDate());
+			for (MPD::SongIterator s = Mpd.CommitSearchSongs(), end; s != end; ++s)
+			{
+				auto key = std::make_tuple(s->getAlbum(), s->getDate());
 				auto it = albums.find(key);
 				if (it == albums.end())
-					albums[key] = s.getMTime();
+					albums[std::move(key)] = s->getMTime();
 				else
-					it->second = s.getMTime();
-			});
-			withUnfilteredMenuReapplyFilter(Albums, [this, &albums, &primary_tag]() {
-				size_t idx = 0;
-				for (auto it = albums.begin(); it != albums.end(); ++it, ++idx)
-				{
-					auto &&entry = AlbumEntry(Album(
-						primary_tag,
-						std::move(std::get<0>(it->first)),
-						std::move(std::get<1>(it->first)),
-						it->second));
-					if (idx < Albums.size())
-					{
-						Albums[idx].value() = entry;
-						Albums[idx].setSeparator(false);
-					}
-					else
-						Albums.addItem(entry);
-				}
+					it->second = std::max(it->second, s->getMTime());
+			};
+			size_t idx = 0;
+			for (const auto &album : albums)
+			{
+				auto entry = AlbumEntry(Album(
+					primary_tag,
+					std::move(std::get<0>(album.first)),
+					std::move(std::get<1>(album.first)),
+					album.second)
+				);
 				if (idx < Albums.size())
-					Albums.resizeList(idx);
-				std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
-				if (albums.size() > 1)
 				{
-					Albums.addSeparator();
-					Albums.addItem(AlbumEntry::mkAllTracksEntry(primary_tag));
+					Albums[idx].value() = std::move(entry);
+					Albums[idx].setSeparator(false);
 				}
-			});
+				else
+					Albums.addItem(std::move(entry));
+				++idx;
+			}
+			if (idx < Albums.size())
+				Albums.resizeList(idx);
+			std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
+			if (albums.size() > 1)
+			{
+				Albums.addSeparator();
+				Albums.addItem(AlbumEntry::mkAllTracksEntry(primary_tag));
+			}
 			Albums.refresh();
 		}
 	}
 	
 	if (!Albums.empty()
-	&& ((Songs.reallyEmpty() && Global::Timer - m_timer > m_fetching_delay) || m_songs_update_request)
+	&& ((Songs.empty() && Global::Timer - m_timer > m_fetching_delay) || m_songs_update_request)
 	)
 	{
-		Songs.clearSearchResults();
 		m_songs_update_request = false;
-		auto &album = Albums.current().value();
+		auto &album = Albums.current()->value();
 		Mpd.StartSearch(true);
 		Mpd.AddSearch(Config.media_lib_primary_tag, album.entry().tag());
 		if (!album.isAllTracksEntry())
@@ -415,73 +423,38 @@ void MediaLibrary::update()
 			Mpd.AddSearch(MPD_TAG_ALBUM, album.entry().album());
 			Mpd.AddSearch(MPD_TAG_DATE, album.entry().date());
 		}
-		withUnfilteredMenuReapplyFilter(Songs, [this, &album]() {
-			size_t idx = 0;
-			Mpd.CommitSearchSongs([this, &idx](MPD::Song s) {
-				bool is_playlist = myPlaylist->checkForSong(s);
-				if (idx < Songs.size())
-				{
-					Songs[idx].value() = s;
-					Songs[idx].setBold(is_playlist);
-				}
-				else
-					Songs.addItem(s, is_playlist);
-				++idx;
-			});
+		size_t idx = 0;
+		for (MPD::SongIterator s = Mpd.CommitSearchSongs(), end; s != end; ++s, ++idx)
+		{
+			bool in_playlist = myPlaylist->checkForSong(*s);
 			if (idx < Songs.size())
-				Songs.resizeList(idx);
-			std::sort(Songs.begin(), Songs.end(), SortSongs(!album.isAllTracksEntry()));
-		});
+			{
+				Songs[idx].value() = std::move(*s);
+				Songs[idx].setBold(in_playlist);
+			}
+			else
+			{
+				auto properties = NC::List::Properties::Selectable;
+				if (in_playlist)
+					properties |= NC::List::Properties::Bold;
+				Songs.addItem(std::move(*s), properties);
+			}
+		};
+		if (idx < Songs.size())
+			Songs.resizeList(idx);
+		std::sort(Songs.begin(), Songs.end(), SortSongs(!album.isAllTracksEntry()));
 		Songs.refresh();
 	}
 }
 
 int MediaLibrary::windowTimeout()
 {
-	if (Albums.reallyEmpty() || Songs.reallyEmpty())
+	if (Albums.empty() || Songs.empty())
 		return m_window_timeout;
 	else
 		return Screen<WindowType>::windowTimeout();
 }
 
-void MediaLibrary::enterPressed()
-{
-	AddToPlaylist(true);
-}
-
-void MediaLibrary::spacePressed()
-{
-	if (Config.space_selects)
-	{
-		if (isActiveWindow(Tags))
-		{
-			size_t idx = Tags.choice();
-			Tags[idx].setSelected(!Tags[idx].isSelected());
-			Tags.scroll(NC::Scroll::Down);
-			Albums.clear();
-			Songs.clear();
-		}
-		else if (isActiveWindow(Albums))
-		{
-			if (!Albums.current().value().isAllTracksEntry())
-			{
-				size_t idx = Albums.choice();
-				Albums[idx].setSelected(!Albums[idx].isSelected());
-				Albums.scroll(NC::Scroll::Down);
-				Songs.clear();
-			}
-		}
-		else if (isActiveWindow(Songs))
-		{
-			size_t idx = Songs.choice();
-			Songs[idx].setSelected(!Songs[idx].isSelected());
-			Songs.scroll(NC::Scroll::Down);
-		}
-	}
-	else
-		AddToPlaylist(false);
-}
-
 void MediaLibrary::mouseButtonPressed(MEVENT me)
 {
 	auto tryNextColumn = [this]() -> bool {
@@ -514,12 +487,7 @@ void MediaLibrary::mouseButtonPressed(MEVENT me)
 		{
 			Tags.Goto(me.y);
 			if (me.bstate & BUTTON3_PRESSED)
-			{
-				size_t pos = Tags.choice();
-				spacePressed();
-				if (pos < Tags.size()-1)
-					Tags.scroll(NC::Scroll::Up);
-			}
+				addItemToPlaylist(false);
 		}
 		else
 			Screen<WindowType>::mouseButtonPressed(me);
@@ -542,12 +510,7 @@ void MediaLibrary::mouseButtonPressed(MEVENT me)
 		{
 			Albums.Goto(me.y);
 			if (me.bstate & BUTTON3_PRESSED)
-			{
-				size_t pos = Albums.choice();
-				spacePressed();
-				if (pos < Albums.size()-1)
-					Albums.scroll(NC::Scroll::Up);
-			}
+				addItemToPlaylist(false);
 		}
 		else
 			Screen<WindowType>::mouseButtonPressed(me);
@@ -560,15 +523,8 @@ void MediaLibrary::mouseButtonPressed(MEVENT me)
 		if (size_t(me.y) < Songs.size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
 		{
 			Songs.Goto(me.y);
-			if (me.bstate & BUTTON1_PRESSED)
-			{
-				size_t pos = Songs.choice();
-				spacePressed();
-				if (pos < Songs.size()-1)
-					Songs.scroll(NC::Scroll::Up);
-			}
-			else
-				enterPressed();
+			bool play = me.bstate & BUTTON3_PRESSED;
+			addItemToPlaylist(play);
 		}
 		else
 			Screen<WindowType>::mouseButtonPressed(me);
@@ -577,181 +533,122 @@ void MediaLibrary::mouseButtonPressed(MEVENT me)
 
 /***********************************************************************/
 
-bool MediaLibrary::allowsFiltering()
-{
-	return true;
-}
-
-std::string MediaLibrary::currentFilter()
-{
-	std::string filter;
-	if (isActiveWindow(Tags))
-		filter = RegexFilter<PrimaryTag>::currentFilter(Tags);
-	else if (isActiveWindow(Albums))
-		filter = RegexItemFilter<AlbumEntry>::currentFilter(Albums);
-	else if (isActiveWindow(Songs))
-		filter = RegexFilter<MPD::Song>::currentFilter(Songs);
-	return filter;
-}
-
-void MediaLibrary::applyFilter(const std::string &filter)
-{
-	if (filter.empty())
-	{
-		if (isActiveWindow(Tags))
-		{
-			Tags.clearFilter();
-			Tags.clearFilterResults();
-		}
-		else if (isActiveWindow(Albums))
-		{
-			Albums.clearFilter();
-			Albums.clearFilterResults();
-		}
-		else if (isActiveWindow(Songs))
-		{
-			Songs.clearFilter();
-			Songs.clearFilterResults();
-		}
-		return;
-	}
-	try
-	{
-		if (isActiveWindow(Tags))
-		{
-			Tags.showAll();
-			auto rx = RegexFilter<PrimaryTag>(
-				boost::regex(filter, Config.regex_type), TagEntryMatcher);
-			Tags.filter(Tags.begin(), Tags.end(), rx);
-		}
-		else if (isActiveWindow(Albums))
-		{
-			Albums.showAll();
-			auto fun = boost::bind(AlbumEntryMatcher, _1, _2, true);
-			auto rx = RegexItemFilter<AlbumEntry>(
-				boost::regex(filter, Config.regex_type), fun);
-			Albums.filter(Albums.begin(), Albums.end(), rx);
-		}
-		else if (isActiveWindow(Songs))
-		{
-			Songs.showAll();
-			auto rx = RegexFilter<MPD::Song>(
-				boost::regex(filter, Config.regex_type), SongEntryMatcher);
-			Songs.filter(Songs.begin(), Songs.end(), rx);
-		}
-	}
-	catch (boost::bad_expression &) { }
-}
-
-/***********************************************************************/
-
 bool MediaLibrary::allowsSearching()
 {
 	return true;
 }
 
-bool MediaLibrary::search(const std::string &constraint)
+void MediaLibrary::setSearchConstraint(const std::string &constraint)
 {
-	if (constraint.empty())
+	if (isActiveWindow(Tags))
 	{
-		if (isActiveWindow(Tags))
-			Tags.clearSearchResults();
-		else if (isActiveWindow(Albums))
-			Albums.clearSearchResults();
-		else if (isActiveWindow(Songs))
-			Songs.clearSearchResults();
-		return false;
+		m_tags_search_predicate = Regex::Filter<PrimaryTag>(
+			Regex::make(constraint, Config.regex_type),
+			TagEntryMatcher
+		);
 	}
-	try
+	else if (isActiveWindow(Albums))
 	{
-		bool result = false;
-		if (isActiveWindow(Tags))
-		{
-			auto rx = RegexFilter<PrimaryTag>(
-				boost::regex(constraint, Config.regex_type), TagEntryMatcher);
-			result = Tags.search(Tags.begin(), Tags.end(), rx);
-		}
-		else if (isActiveWindow(Albums))
-		{
-			auto fun = boost::bind(AlbumEntryMatcher, _1, _2, false);
-			auto rx = RegexItemFilter<AlbumEntry>(
-				boost::regex(constraint, Config.regex_type), fun);
-			result = Albums.search(Albums.begin(), Albums.end(), rx);
-		}
-		else if (isActiveWindow(Songs))
-		{
-			auto rx = RegexFilter<MPD::Song>(
-				boost::regex(constraint, Config.regex_type), SongEntryMatcher);
-			result = Songs.search(Songs.begin(), Songs.end(), rx);
-		}
-		return result;
+		m_albums_search_predicate = Regex::ItemFilter<AlbumEntry>(
+			Regex::make(constraint, Config.regex_type),
+			std::bind(AlbumEntryMatcher, ph::_1, ph::_2, false)
+		);
 	}
-	catch (boost::bad_expression &)
+	else if (isActiveWindow(Songs))
 	{
-		return false;
+		m_songs_search_predicate = Regex::Filter<MPD::Song>(
+			Regex::make(constraint, Config.regex_type),
+			SongEntryMatcher
+		);
 	}
 }
 
-void MediaLibrary::nextFound(bool wrap)
+void MediaLibrary::clearConstraint()
 {
 	if (isActiveWindow(Tags))
-		Tags.nextFound(wrap);
+		m_tags_search_predicate.clear();
 	else if (isActiveWindow(Albums))
-		Albums.nextFound(wrap);
+		m_albums_search_predicate.clear();
 	else if (isActiveWindow(Songs))
-		Songs.nextFound(wrap);
+		m_songs_search_predicate.clear();
 }
 
-void MediaLibrary::prevFound(bool wrap)
+bool MediaLibrary::find(SearchDirection direction, bool wrap, bool skip_current)
 {
+	bool result = false;
 	if (isActiveWindow(Tags))
-		Tags.prevFound(wrap);
+		result = search(Tags, m_tags_search_predicate, direction, wrap, skip_current);
 	else if (isActiveWindow(Albums))
-		Albums.prevFound(wrap);
+		result = search(Albums, m_albums_search_predicate, direction, wrap, skip_current);
 	else if (isActiveWindow(Songs))
-		Songs.prevFound(wrap);
+		result = search(Songs, m_songs_search_predicate, direction, wrap, skip_current);
+	return result;
 }
 
 /***********************************************************************/
 
-ProxySongList MediaLibrary::proxySongList()
-{
-	auto ptr = ProxySongList();
-	if (isActiveWindow(Songs))
-		ptr = songsProxyList();
-	return ptr;
-}
-
-bool MediaLibrary::allowsSelection()
-{
-	return true;
-}
-
-void MediaLibrary::reverseSelection()
+bool MediaLibrary::itemAvailable()
 {
 	if (isActiveWindow(Tags))
-		reverseSelectionHelper(Tags.begin(), Tags.end());
-	else if (isActiveWindow(Albums))
-	{
-		// omit "All tracks"
-		if (Albums.size() > 1)
-			reverseSelectionHelper(Albums.begin(), Albums.end()-2);
-		else
-			reverseSelectionHelper(Albums.begin(), Albums.end());
-	}
-	else if (isActiveWindow(Songs))
-		reverseSelectionHelper(Songs.begin(), Songs.end());
+		return !Tags.empty();
+	if (isActiveWindow(Albums))
+		return !Albums.empty();
+	if (isActiveWindow(Songs))
+		return !Songs.empty();
+	return false;
 }
 
-MPD::SongList MediaLibrary::getSelectedSongs()
+bool MediaLibrary::addItemToPlaylist(bool play)
 {
-	MPD::SongList result;
+	bool result = false;
+	if (isActiveWindow(Songs))
+		result = addSongToPlaylist(Songs.current()->value(), play);
+	else
+	{
+		if (isActiveWindow(Tags)
+		||  (isActiveWindow(Albums) && Albums.current()->value().isAllTracksEntry()))
+		{
+			Mpd.StartSearch(true);
+			Mpd.AddSearch(Config.media_lib_primary_tag, Tags.current()->value().tag());
+			std::vector<MPD::Song> list(
+				std::make_move_iterator(Mpd.CommitSearchSongs()),
+				std::make_move_iterator(MPD::SongIterator())
+			);
+			result = addSongsToPlaylist(list.begin(), list.end(), play, -1);
+			std::string tag_type = boost::locale::to_lower(
+				tagTypeToString(Config.media_lib_primary_tag));
+			Statusbar::printf("Songs with %1% \"%2%\" added%3%",
+				tag_type, Tags.current()->value().tag(), withErrors(result)
+			);
+		}
+		else if (isActiveWindow(Albums))
+		{
+			std::vector<MPD::Song> list(
+				std::make_move_iterator(getSongsFromAlbum(Albums.current()->value())),
+				std::make_move_iterator(MPD::SongIterator())
+			);
+			result = addSongsToPlaylist(list.begin(), list.end(), play, -1);
+			Statusbar::printf("Songs from album \"%1%\" added%2%",
+				Albums.current()->value().entry().album(), withErrors(result)
+			);
+		}
+	}
+	return result;
+}
+
+std::vector<MPD::Song> MediaLibrary::getSelectedSongs()
+{
+	std::vector<MPD::Song> result;
 	if (isActiveWindow(Tags))
 	{
 		auto tag_handler = [&result](const std::string &tag) {
 			Mpd.StartSearch(true);
 			Mpd.AddSearch(Config.media_lib_primary_tag, tag);
-			Mpd.CommitSearchSongs(vectorMoveInserter(result));
+			std::copy(
+				std::make_move_iterator(Mpd.CommitSearchSongs()),
+				std::make_move_iterator(MPD::SongIterator()),
+				std::back_inserter(result)
+			);
 		};
 		bool any_selected = false;
 		for (auto &e : Tags)
@@ -764,7 +661,7 @@ MPD::SongList MediaLibrary::getSelectedSongs()
 		}
 		// if no item is selected, add current one
 		if (!any_selected && !Tags.empty())
-			tag_handler(Tags.current().value().tag());
+			tag_handler(Tags.current()->value().tag());
 	}
 	else if (isActiveWindow(Albums))
 	{
@@ -780,11 +677,15 @@ MPD::SongList MediaLibrary::getSelectedSongs()
 					Mpd.AddSearch(Config.media_lib_primary_tag, sc.entry().tag());
 				else
 					Mpd.AddSearch(Config.media_lib_primary_tag,
-					              Tags.current().value().tag());
+					              Tags.current()->value().tag());
 					Mpd.AddSearch(MPD_TAG_ALBUM, sc.entry().album());
 				Mpd.AddSearch(MPD_TAG_DATE, sc.entry().date());
 				size_t begin = result.size();
-				Mpd.CommitSearchSongs(vectorMoveInserter(result));
+				std::copy(
+					std::make_move_iterator(Mpd.CommitSearchSongs()),
+					std::make_move_iterator(MPD::SongIterator()),
+					std::back_inserter(result)
+				);
 				std::sort(result.begin()+begin, result.end(), SortSongs(false));
 			}
 		}
@@ -792,19 +693,16 @@ MPD::SongList MediaLibrary::getSelectedSongs()
 		if (!any_selected && !Albums.empty())
 		{
 			size_t begin = result.size();
-			withSongsFromAlbum(Albums.current().value().entry(), vectorMoveInserter(result));
+			std::copy(
+				std::make_move_iterator(getSongsFromAlbum(Albums.current()->value().entry())),
+				std::make_move_iterator(MPD::SongIterator()),
+				std::back_inserter(result)
+			);
 			std::sort(result.begin()+begin, result.end(), SortSongs(false));
 		}
 	}
 	else if (isActiveWindow(Songs))
-	{
-		for (auto it = Songs.begin(); it != Songs.end(); ++it)
-			if (it->isSelected())
-				result.push_back(it->value());
-		// if no item is selected, add current one
-		if (result.empty() && !Songs.empty())
-			result.push_back(Songs.current().value());
-	}
+		Songs.getSelectedSongs();
 	return result;
 }
 
@@ -815,12 +713,12 @@ bool MediaLibrary::previousColumnAvailable()
 	assert(!hasTwoColumns || !isActiveWindow(Tags));
 	if (isActiveWindow(Songs))
 	{
-		if (!Albums.reallyEmpty() && (hasTwoColumns || !Tags.reallyEmpty()))
+		if (!Albums.empty() && (hasTwoColumns || !Tags.empty()))
 			return true;
 	}
 	else if (isActiveWindow(Albums))
 	{
-		if (!hasTwoColumns && !Tags.reallyEmpty())
+		if (!hasTwoColumns && !Tags.empty())
 			return true;
 	}
 	return false;
@@ -849,12 +747,12 @@ bool MediaLibrary::nextColumnAvailable()
 	assert(!hasTwoColumns || !isActiveWindow(Tags));
 	if (isActiveWindow(Tags))
 	{
-		if (!Albums.reallyEmpty() && !Songs.reallyEmpty())
+		if (!Albums.empty() && !Songs.empty())
 			return true;
 	}
 	else if (isActiveWindow(Albums))
 	{
-		if (!Songs.reallyEmpty())
+		if (!Songs.empty())
 			return true;
 	}
 	return false;
@@ -917,13 +815,6 @@ int MediaLibrary::Columns()
 		return 3;
 }
 
-ProxySongList MediaLibrary::songsProxyList()
-{
-	return ProxySongList(Songs, [](NC::Menu<MPD::Song>::Item &item) {
-		return &item.value();
-	});
-}
-
 void MediaLibrary::toggleSortMode()
 {
 	Config.media_library_sort_by_mtime = !Config.media_library_sort_by_mtime;
@@ -931,9 +822,7 @@ void MediaLibrary::toggleSortMode()
 		Config.media_library_sort_by_mtime ? "modification time" : "name");
 	if (hasTwoColumns)
 	{
-		withUnfilteredMenuReapplyFilter(Albums, [this]() {
-			std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
-		});
+		std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
 		Albums.refresh();
 		Songs.clear();
 		if (Config.titles_visibility)
@@ -946,17 +835,14 @@ void MediaLibrary::toggleSortMode()
 	}
 	else
 	{
-		withUnfilteredMenuReapplyFilter(Tags, [this]() {
-			// if we already have modification times,
-			// just resort. otherwise refetch the list.
-			if (!Tags.empty() && Tags[0].value().mtime() > 0)
-			{
-				std::sort(Tags.beginV(), Tags.endV(), SortPrimaryTags());
-				Tags.refresh();
-			}
-			else
-				Tags.clear();
-		});
+		// if we already have modification times, just resort. otherwise refetch the list.
+		if (!Tags.empty() && Tags[0].value().mtime() > 0)
+		{
+			std::sort(Tags.beginV(), Tags.endV(), SortPrimaryTags());
+			Tags.refresh();
+		}
+		else
+			Tags.clear();
 		Albums.clear();
 		Songs.clear();
 	}
@@ -973,6 +859,7 @@ void MediaLibrary::LocateSong(const MPD::Song &s)
 		Statusbar::printf("Can't use this function because the song has no %s tag", item_type);
 		return;
 	}
+
 	if (!s.isFromDatabase())
 	{
 		Statusbar::print("Song is not from the database");
@@ -983,118 +870,88 @@ void MediaLibrary::LocateSong(const MPD::Song &s)
 		switchTo();
 	Statusbar::put() << "Jumping to song...";
 	Global::wFooter->refresh();
-	
+
 	if (!hasTwoColumns)
 	{
-		Tags.showAll();
 		if (Tags.empty())
 			update();
-		if (primary_tag != Tags.current().value().tag())
+
+		if (!MoveToTag(Tags, primary_tag))
 		{
-			for (size_t i = 0; i < Tags.size(); ++i)
-			{
-				if (primary_tag == Tags[i].value().tag())
-				{
-					Tags.highlight(i);
-					Albums.clear();
-					Songs.clear();
-					break;
-				}
-			}
+			// The tag could not be found. Since this was called from an existing
+			// song, the tag should exist in the library, but it was not listed by
+			// list/listallinfo. This is the case with some players where it is not
+			// possible to list all of the library, e.g. mopidy with mopidy-spotify.
+			// To workaround this we simply insert the missing tag.
+			Tags.addItem(PrimaryTag(primary_tag, s.getMTime()));
+			std::sort(Tags.beginV(), Tags.endV(), SortPrimaryTags());
+			Tags.refresh();
+			MoveToTag(Tags, primary_tag);
 		}
+		Albums.clear();
 	}
 	
-	Albums.showAll();
 	if (Albums.empty())
 		update();
-	
-	std::string album = s.getAlbum();
-	std::string date = s.getDate();
-	if ((hasTwoColumns && Albums.current().value().entry().tag() != primary_tag)
-	||  album != Albums.current().value().entry().album()
-	||  date != Albums.current().value().entry().date())
+
+	// When you locate a song in the media library, if no albums or no songs
+	// are found, set the active column to the previous one (tags if no albums,
+	// and albums if no songs). This makes sure that the active column is not
+	// empty, which may make it impossible to move out of.
+	//
+	// The problem was if you highlight some song in the rightmost column in
+	// the media browser and then go to some other window and select locate
+	// song. If the tag or album it looked up in the media library was
+	// empty, the selection would stay in the songs column while it was empty.
+	// This made the selection impossible to change.
+	//
+	// This only is a problem if a song has some tag or album for which the
+	// find command doesn't return any results. This should never really happen
+	// unless there is some inconsistency in the player. However, it may
+	// happen, so we need to handle it.
+	//
+	// Note: We don't want to return when no albums are found in two column
+	// mode. In this case, we try to insert the album, as we do with tags when
+	// they are not found.
+	if (hasTwoColumns || !Albums.empty())
 	{
-		for (size_t i = 0; i < Albums.size(); ++i)
+		if (!MoveToAlbum(Albums, primary_tag, s))
 		{
-			if ((!hasTwoColumns || Albums[i].value().entry().tag() == primary_tag)
-			&&   album == Albums[i].value().entry().album()
-			&&   date == Albums[i].value().entry().date())
-			{
-				Albums.highlight(i);
-				Songs.clear();
-				break;
-			}
+			// The album could not be found, insert it if in two column mode.
+			// See comment about tags not found above. This is the equivalent
+			// for two column mode.
+			Albums.addItem(AlbumEntry(
+				Album(primary_tag, s.getAlbum(), s.getDate(), s.getMTime())
+			));
+			std::sort(Albums.beginV(), Albums.endV(), SortAlbumEntries());
+			Albums.refresh();
+			MoveToAlbum(Albums, primary_tag, s);
 		}
-	}
-	
-	Songs.showAll();
-	if (Songs.empty())
+
+		Songs.clear();
 		update();
-	
-	if (s != Songs.current().value())
-	{
-		for (size_t i = 0; i < Songs.size(); ++i)
+
+		if (!Songs.empty())
 		{
-			if (s == Songs[i].value())
+			if (s != Songs.current()->value())
 			{
-				Songs.highlight(i);
-				break;
+				auto begin = Songs.beginV(), end = Songs.endV();
+				auto it = std::find(begin, end, s);
+				if (it != end)
+					Songs.highlight(it-begin);
 			}
+			nextColumn();
+			nextColumn();
 		}
+		else // invalid album was added, clear the list
+			Albums.clear();
 	}
-	
-	Tags.setHighlightColor(Config.main_highlight_color);
-	Albums.setHighlightColor(Config.main_highlight_color);
-	Songs.setHighlightColor(Config.active_column_color);
-	w = &Songs;
+	else // invalid tag was added, clear the list
+		Tags.clear();
 	refresh();
 }
 
-void MediaLibrary::AddToPlaylist(bool add_n_play)
-{
-	if (isActiveWindow(Songs) && !Songs.empty())
-		addSongToPlaylist(Songs.current().value(), add_n_play);
-	else
-	{
-		if ((!Tags.empty() && isActiveWindow(Tags))
-		||  (isActiveWindow(Albums) && Albums.current().value().isAllTracksEntry()))
-		{
-			MPD::SongList list;
-			Mpd.StartSearch(true);
-			Mpd.AddSearch(Config.media_lib_primary_tag, Tags.current().value().tag());
-			Mpd.CommitSearchSongs(vectorMoveInserter(list));
-			bool success = addSongsToPlaylist(list.begin(), list.end(), add_n_play, -1);
-			std::string tag_type = boost::locale::to_lower(
-				tagTypeToString(Config.media_lib_primary_tag));
-			Statusbar::printf("Songs with %1% \"%2%\" added%3%",
-				tag_type, Tags.current().value().tag(), withErrors(success)
-			);
-		}
-		else if (isActiveWindow(Albums))
-		{
-			MPD::SongList list;
-			withSongsFromAlbum(Albums.current().value(), vectorMoveInserter(list));
-			bool success = addSongsToPlaylist(list.begin(), list.end(), add_n_play, -1);
-			Statusbar::printf("Songs from album \"%1%\" added%2%",
-				Albums.current().value().entry().album(), withErrors(success)
-			);
-		}
-	}
-	
-	if (!add_n_play)
-	{
-		w->scroll(NC::Scroll::Down);
-		if (isActiveWindow(Tags))
-		{
-			Albums.clear();
-			Songs.clear();
-		}
-		else if (isActiveWindow(Albums))
-			Songs.clear();
-	}
-}
-
-namespace {//
+namespace {
 
 std::string AlbumToString(const AlbumEntry &ae)
 {
@@ -1120,24 +977,77 @@ std::string AlbumToString(const AlbumEntry &ae)
 
 std::string SongToString(const MPD::Song &s)
 {
-	return s.toString(Config.song_library_format, Config.tags_separator);
+	return Format::stringify<char>(
+		Config.song_library_format, &s
+	);
 }
 
-bool TagEntryMatcher(const boost::regex &rx, const MediaLibrary::PrimaryTag &pt)
+bool TagEntryMatcher(const Regex::Regex &rx, const PrimaryTag &pt)
 {
-	return boost::regex_search(pt.tag(), rx);
+	return Regex::search(pt.tag(), rx);
 }
 
-bool AlbumEntryMatcher(const boost::regex &rx, const NC::Menu<AlbumEntry>::Item &item, bool filter)
+bool AlbumEntryMatcher(const Regex::Regex &rx, const NC::Menu<AlbumEntry>::Item &item, bool filter)
 {
 	if (item.isSeparator() || item.value().isAllTracksEntry())
 		return filter;
-	return boost::regex_search(AlbumToString(item.value()), rx);
+	return Regex::search(AlbumToString(item.value()), rx);
 }
 
-bool SongEntryMatcher(const boost::regex &rx, const MPD::Song &s)
+bool SongEntryMatcher(const Regex::Regex &rx, const MPD::Song &s)
 {
-	return boost::regex_search(SongToString(s), rx);
+	return Regex::search(SongToString(s), rx);
+}
+
+bool MoveToTag(NC::Menu<PrimaryTag> &tags, const std::string &primary_tag)
+{
+	if (tags.empty())
+		return false;
+
+	auto equals_fun_argument = [&](PrimaryTag &e) {
+		return e.tag() == primary_tag;
+	};
+
+	if (equals_fun_argument(*tags.currentV()))
+		return true;
+
+	auto begin = tags.beginV(), end = tags.endV();
+	auto it = std::find_if(begin, end, equals_fun_argument);
+	if (it != end)
+	{
+		tags.highlight(it-begin);
+		return true;
+	}
+
+	return false;
+}
+
+bool MoveToAlbum(NC::Menu<AlbumEntry> &albums, const std::string &primary_tag, const MPD::Song &s)
+{
+	if (albums.empty())
+		return false;
+
+	std::string album = s.getAlbum();
+	std::string date = s.getDate();
+
+	auto equals_fun_argument = [&](AlbumEntry &e) {
+		return (!hasTwoColumns || e.entry().tag() == primary_tag)
+		    && e.entry().album() == album
+		    && e.entry().date() == date;
+	};
+
+	if (equals_fun_argument(*albums.currentV()))
+		return true;
+
+	auto begin = albums.beginV(), end = albums.endV();
+	auto it = std::find_if(begin, end, equals_fun_argument);
+	if (it != end)
+	{
+		albums.highlight(it-begin);
+		return true;
+	}
+
+	return false;
 }
 
 }
