@@ -37,7 +37,6 @@
 #include "charset.h"
 #include "configuration.h"
 #include "global.h"
-#include "error.h"
 #include "helpers.h"
 #include "lyrics.h"
 #include "outputs.h"
@@ -49,20 +48,17 @@
 #include "title.h"
 #include "utility/conversion.h"
 
+namespace ph = std::placeholders;
+
 namespace
 {
 	std::ofstream errorlog;
 	std::streambuf *cerr_buffer;
 	bool run_resize_screen = false;
 	
-#	if !defined(WIN32)
 	void sighandler(int sig)
 	{
-		if (sig == SIGPIPE)
-		{
-			Statusbar::print("SIGPIPE (broken pipe signal) received");
-		}
-		else if (sig == SIGWINCH)
+		if (sig == SIGWINCH)
 		{
 			run_resize_screen = true;
 		}
@@ -71,17 +67,14 @@ namespace
 		signal(sig, sighandler);
 #		endif // __sun && __SVR4
 	}
-#	endif // !WIN32
-	
+
 	void do_at_exit()
 	{
 		// restore old cerr buffer
 		std::cerr.rdbuf(cerr_buffer);
 		errorlog.close();
 		Mpd.Disconnect();
-#		ifndef USE_PDCURSES // destroying screen somehow crashes pdcurses
 		NC::destroyScreen();
-#		endif // USE_PDCURSES
 		windowTitle("");
 	}
 }
@@ -118,7 +111,7 @@ int main(int argc, char **argv)
 	sigignore(SIGINT);
 #	endif // !WIN32
 
-	NC::initScreen("ncmpcpp ver. " VERSION, Config.colors_enabled);
+	NC::initScreen(Config.colors_enabled, Config.mouse_support);
 	
 	Actions::OriginalStatusbarVisibility = Config.statusbar_visibility;
 
@@ -129,29 +122,49 @@ int main(int argc, char **argv)
 	Actions::validateScreenSize();
 	Actions::initializeScreens();
 	
-	wHeader = new NC::Window(0, 0, COLS, Actions::HeaderHeight, "", Config.header_color, NC::Border::None);
+	wHeader = new NC::Window(0, 0, COLS, Actions::HeaderHeight, "", Config.header_color, NC::Border());
 	if (Config.header_visibility || Config.design == Design::Alternative)
 		wHeader->display();
 	
-	wFooter = new NC::Window(0, Actions::FooterStartY, COLS, Actions::FooterHeight, "", Config.statusbar_color, NC::Border::None);
-	wFooter->setGetStringHelper(Statusbar::Helpers::getString);
+	wFooter = new NC::Window(0, Actions::FooterStartY, COLS, Actions::FooterHeight, "", Config.statusbar_color, NC::Border());
+	wFooter->setPromptHook(Statusbar::Helpers::mainHook);
 	
 	// initialize global timer
 	Timer = boost::posix_time::microsec_clock::local_time();
 	
 	// initialize playlist
 	myPlaylist->switchTo();
-	
+
+	// go to startup screen
+	if (Config.startup_screen_type != myScreen->type())
+	{
+		auto startup_screen = toScreen(Config.startup_screen_type);
+		assert(startup_screen != nullptr);
+		startup_screen->switchTo();
+	}
+
+	// lock current screen and go to the slave one if applicable
+	if (Config.startup_slave_screen_type)
+	{
+		auto slave_screen_type = *Config.startup_slave_screen_type;
+		bool screen_locked = myScreen->lock();
+		if (screen_locked && slave_screen_type != myScreen->type())
+		{
+			auto slave_screen = toScreen(slave_screen_type);
+			assert(slave_screen != nullptr);
+			slave_screen->switchTo();
+			if (!Config.startup_slave_screen_focus)
+				Actions::get(Actions::Type::MasterScreen).execute();
+		}
+	}
+
 	// local variables
 	bool key_pressed = false;
-	Key input = Key::noOp;
+	auto input = NC::Key::None;
 	auto connect_attempt = boost::posix_time::from_time_t(0);
-	auto past = boost::posix_time::from_time_t(0);
-	
-	/// enable mouse
-	mouseinterval(0);
-	if (Config.mouse_support)
-		mousemask(ALL_MOUSE_EVENTS, 0);
+	auto update_environment = static_cast<Actions::UpdateEnvironment &>(
+		Actions::get(Actions::Type::UpdateEnvironment)
+	);
 	
 	while (!Actions::ExitMainLoop)
 	{
@@ -178,33 +191,22 @@ int main(int argc, char **argv)
 					Status::handleClientError(e);
 				}
 			}
-			
-			// update timer, status if necessary etc.
-			Status::trace(!key_pressed, true);
 
 			if (run_resize_screen)
 			{
 				Actions::resizeScreen(true);
 				run_resize_screen = false;
 			}
-			
-			// header stuff
-			if ((myScreen == myPlaylist || myScreen == myBrowser || myScreen == myLyrics)
-			&&  (Timer - past > boost::posix_time::milliseconds(500))
-			)
-			{
-				drawHeader();
-				past = Timer;
-			}
-			
-			if (key_pressed)
-				myScreen->refreshWindow();
-			input = Key::read(*wFooter);
-			key_pressed = input != Key::noOp;
-			
+
+			update_environment.run(!key_pressed, key_pressed);
+
+			input = readKey(*wFooter);
+			key_pressed = input != NC::Key::None;
 			if (!key_pressed)
 				continue;
 
+			//Statusbar::print(ToString(keyToWString(input)));
+
 			// The reason we want to update timer here is that if the timer is updated
 			// in Status::trace, then Key::read usually blocks for 500ms and if key is
 			// pressed 400ms after Key::read was called, we end up with Timer that is
@@ -216,19 +218,23 @@ int main(int argc, char **argv)
 			try
 			{
 				auto k = Bindings.get(input);
-				std::any_of(k.first, k.second, boost::bind(&Binding::execute, _1));
+				std::any_of(k.first, k.second, std::bind(&Binding::execute, ph::_1));
 			}
 			catch (ConversionError &e)
 			{
-				Statusbar::printf("Couldn't convert value \"%1%\" to target type", e.value());
+				Statusbar::printf("Invalid value: %1%", e.value());
 			}
 			catch (OutOfBounds &e)
 			{
 				Statusbar::printf("Error: %1%", e.errorMessage());
 			}
+			catch (NC::PromptAborted &)
+			{
+				Statusbar::printf("Action aborted");
+			}
 
 			if (myScreen == myPlaylist)
-				myPlaylist->EnableHighlighting();
+				myPlaylist->enableHighlighting();
 		}
 		catch (MPD::ClientError &e)
 		{
