@@ -43,17 +43,14 @@ struct Outputs: Screen<NC::Menu<MPD::Output>>, Tabbable
 	
 	virtual void update() OVERRIDE { }
 	
-	virtual void enterPressed() OVERRIDE;
-	virtual void spacePressed() OVERRIDE { }
 	virtual void mouseButtonPressed(MEVENT me) OVERRIDE;
 	
+	virtual bool isLockable() OVERRIDE { return true; }
 	virtual bool isMergable() OVERRIDE { return true; }
 	
 	// private members
-	void FetchList();
-	
-protected:
-	virtual bool isLockable() OVERRIDE { return true; }
+	void fetchList();
+	void toggleOutput();
 };
 
 extern Outputs *myOutputs;
