@@ -25,14 +25,16 @@
 #include <unordered_map>
 
 #include "interfaces.h"
+#include "regex_filter.h"
 #include "screen.h"
 #include "song.h"
+#include "song_list.h"
 
-struct Playlist: Screen<NC::Menu<MPD::Song>>, Filterable, HasSongs, Searchable, Tabbable
+struct Playlist: Screen<SongMenu>, HasSongs, Searchable, Tabbable
 {
 	Playlist();
 	
-	// Screen<NC::Menu<MPD::Song>> implementation
+	// Screen<SongMenu> implementation
 	virtual void switchTo() OVERRIDE;
 	virtual void resize() OVERRIDE;
 	
@@ -41,39 +43,28 @@ struct Playlist: Screen<NC::Menu<MPD::Song>>, Filterable, HasSongs, Searchable,
 	
 	virtual void update() OVERRIDE;
 	
-	virtual void enterPressed() OVERRIDE;
-	virtual void spacePressed() OVERRIDE;
 	virtual void mouseButtonPressed(MEVENT me) OVERRIDE;
 	
+	virtual bool isLockable() OVERRIDE { return true; }
 	virtual bool isMergable() OVERRIDE { return true; }
 	
-	// Filterable implementation
-	virtual bool allowsFiltering() OVERRIDE;
-	virtual std::string currentFilter() OVERRIDE;
-	virtual void applyFilter(const std::string &filter) OVERRIDE;
-	
 	// Searchable implementation
-	virtual bool allowsSearching();
-	virtual bool search(const std::string &constraint) OVERRIDE;
-	virtual void nextFound(bool wrap) OVERRIDE;
-	virtual void prevFound(bool wrap) OVERRIDE;
+	virtual bool allowsSearching() OVERRIDE;
+	virtual void setSearchConstraint(const std::string &constraint) OVERRIDE;
+	virtual void clearConstraint() OVERRIDE;
+	virtual bool find(SearchDirection direction, bool wrap, bool skip_current) OVERRIDE;
 	
 	// HasSongs implementation
-	virtual ProxySongList proxySongList() OVERRIDE;
-	
-	virtual bool allowsSelection() OVERRIDE;
-	virtual void reverseSelection() OVERRIDE;
-	virtual MPD::SongList getSelectedSongs() OVERRIDE;
+	virtual bool itemAvailable() OVERRIDE;
+	virtual bool addItemToPlaylist(bool play) OVERRIDE;
+	virtual std::vector<MPD::Song> getSelectedSongs() OVERRIDE;
 	
 	// private members
 	MPD::Song nowPlayingSong();
+
+	void enableHighlighting();
 	
-	bool isFiltered();
-	void Reverse();
-	
-	void EnableHighlighting();
-	
-	void SetSelectedItemsPriority(int prio);
+	void setSelectedItemsPriority(int prio);
 	
 	bool checkForSong(const MPD::Song &s);
 	void registerSong(const MPD::Song &s);
@@ -82,9 +73,6 @@ struct Playlist: Screen<NC::Menu<MPD::Song>>, Filterable, HasSongs, Searchable,
 	void reloadTotalLength() { m_reload_total_length = true; }
 	void reloadRemaining() { m_reload_remaining = true; }
 	
-protected:
-	virtual bool isLockable() OVERRIDE { return true; }
-	
 private:
 	std::string getTotalLength();
 
@@ -100,6 +88,8 @@ private:
 
 	bool m_reload_total_length;
 	bool m_reload_remaining;
+
+	Regex::Filter<MPD::Song> m_search_predicate;
 };
 
 extern Playlist *myPlaylist;
