@@ -19,7 +19,7 @@
  ***************************************************************************/
 
 #include <algorithm>
-#include <boost/bind.hpp>
+#include <boost/lambda/bind.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <cassert>
 
@@ -29,11 +29,13 @@
 #include "helpers.h"
 #include "playlist.h"
 #include "playlist_editor.h"
+#include "menu_impl.h"
 #include "mpdpp.h"
-#include "regex_filter.h"
 #include "status.h"
 #include "statusbar.h"
 #include "tag_editor.h"
+#include "helpers/song_iterator_maker.h"
+#include "utility/functional.h"
 #include "utility/comparators.h"
 #include "title.h"
 #include "screen_switcher.h"
@@ -41,6 +43,8 @@
 using Global::MainHeight;
 using Global::MainStartY;
 
+namespace ph = std::placeholders;
+
 PlaylistEditor *myPlaylistEditor;
 
 namespace {
@@ -51,31 +55,31 @@ size_t RightColumnStartX;
 size_t RightColumnWidth;
 
 std::string SongToString(const MPD::Song &s);
-bool PlaylistEntryMatcher(const boost::regex &rx, const std::string &playlist);
-bool SongEntryMatcher(const boost::regex &rx, const MPD::Song &s);
+bool PlaylistEntryMatcher(const Regex::Regex &rx, const MPD::Playlist &playlist);
+bool SongEntryMatcher(const Regex::Regex &rx, const MPD::Song &s);
 
 }
 
 PlaylistEditor::PlaylistEditor()
 : m_timer(boost::posix_time::from_time_t(0))
-, m_window_timeout(Config.data_fetching_delay ? 250 : 500)
+, m_window_timeout(Config.data_fetching_delay ? 250 : BaseScreen::defaultWindowTimeout)
 , m_fetching_delay(boost::posix_time::milliseconds(Config.data_fetching_delay ? 250 : -1))
 {
 	LeftColumnWidth = COLS/3-1;
 	RightColumnStartX = LeftColumnWidth+1;
 	RightColumnWidth = COLS-LeftColumnWidth-1;
 	
-	Playlists = NC::Menu<std::string>(0, MainStartY, LeftColumnWidth, MainHeight, Config.titles_visibility ? "Playlists" : "", Config.main_color, NC::Border::None);
+	Playlists = NC::Menu<MPD::Playlist>(0, MainStartY, LeftColumnWidth, MainHeight, Config.titles_visibility ? "Playlists" : "", Config.main_color, NC::Border());
 	Playlists.setHighlightColor(Config.active_column_color);
 	Playlists.cyclicScrolling(Config.use_cyclic_scrolling);
 	Playlists.centeredCursor(Config.centered_cursor);
 	Playlists.setSelectedPrefix(Config.selected_item_prefix);
 	Playlists.setSelectedSuffix(Config.selected_item_suffix);
-	Playlists.setItemDisplayer([](NC::Menu<std::string> &menu) {
-		menu << Charset::utf8ToLocale(menu.drawn()->value());
+	Playlists.setItemDisplayer([](NC::Menu<MPD::Playlist> &menu) {
+		menu << Charset::utf8ToLocale(menu.drawn()->value().path());
 	});
 	
-	Content = NC::Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, Config.titles_visibility ? "Playlist content" : "", Config.main_color, NC::Border::None);
+	Content = NC::Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, Config.titles_visibility ? "Playlist content" : "", Config.main_color, NC::Border());
 	Content.setHighlightColor(Config.main_highlight_color);
 	Content.cyclicScrolling(Config.use_cyclic_scrolling);
 	Content.centeredCursor(Config.centered_cursor);
@@ -84,10 +88,14 @@ PlaylistEditor::PlaylistEditor()
 	switch (Config.playlist_editor_display_mode)
 	{
 		case DisplayMode::Classic:
-			Content.setItemDisplayer(boost::bind(Display::Songs, _1, contentProxyList(), Config.song_list_format));
+			Content.setItemDisplayer(std::bind(
+				Display::Songs, ph::_1, std::cref(Content), std::cref(Config.song_list_format)
+			));
 			break;
 		case DisplayMode::Columns:
-			Content.setItemDisplayer(boost::bind(Display::SongsInColumns, _1, contentProxyList()));
+			Content.setItemDisplayer(std::bind(
+				Display::SongsInColumns, ph::_1, std::cref(Content)
+			));
 			break;
 	}
 	
@@ -128,35 +136,32 @@ void PlaylistEditor::refresh()
 void PlaylistEditor::switchTo()
 {
 	SwitchTo::execute(this);
-	markSongsInPlaylist(contentProxyList());
+	markSongsInPlaylist(Content);
 	drawHeader();
 	refresh();
 }
 
 void PlaylistEditor::update()
 {
-	if (Playlists.reallyEmpty() || m_playlists_update_requested)
+	if (Playlists.empty() || m_playlists_update_requested)
 	{
 		m_playlists_update_requested = false;
-		Playlists.clearSearchResults();
-		withUnfilteredMenuReapplyFilter(Playlists, [this]() {
-			size_t idx = 0;
-			Mpd.GetPlaylists([this, &idx](std::string playlist) {
-				if (idx < Playlists.size())
-					Playlists[idx].value() = playlist;
-				else
-					Playlists.addItem(playlist);
-				++idx;
-			});
+		size_t idx = 0;
+		for (MPD::PlaylistIterator it = Mpd.GetPlaylists(), end; it != end; ++it, ++idx)
+		{
 			if (idx < Playlists.size())
-				Playlists.resizeList(idx);
-			std::sort(Playlists.beginV(), Playlists.endV(),
-				LocaleBasedSorting(std::locale(), Config.ignore_leading_the));
-		});
+				Playlists[idx].value() = std::move(*it);
+			else
+				Playlists.addItem(std::move(*it));
+		};
+		if (idx < Playlists.size())
+			Playlists.resizeList(idx);
+		std::sort(Playlists.beginV(), Playlists.endV(),
+			LocaleBasedSorting(std::locale(), Config.ignore_leading_the));
 		Playlists.refresh();
 	}
 	
-	if ((Content.reallyEmpty() && Global::Timer - m_timer > m_fetching_delay)
+	if ((Content.empty() && Global::Timer - m_timer > m_fetching_delay)
 	||  m_content_update_requested)
 	{
 		m_content_update_requested = false;
@@ -164,48 +169,48 @@ void PlaylistEditor::update()
 			Content.clear();
 		else
 		{
-			Content.clearSearchResults();
-			withUnfilteredMenuReapplyFilter(Content, [this]() {
-				size_t idx = 0;
-				Mpd.GetPlaylistContent(Playlists.current().value(), [this, &idx](MPD::Song s) {
-					if (idx < Content.size())
-					{
-						Content[idx].value() = s;
-						Content[idx].setBold(myPlaylist->checkForSong(s));
-					}
-					else
-						Content.addItem(s, myPlaylist->checkForSong(s));
-					++idx;
-				});
+			size_t idx = 0;
+			MPD::SongIterator s = Mpd.GetPlaylistContent(Playlists.current()->value().path()), end;
+			for (; s != end; ++s, ++idx)
+			{
+				bool in_playlist = myPlaylist->checkForSong(*s);
 				if (idx < Content.size())
-					Content.resizeList(idx);
-				std::string title;
-				if (Config.titles_visibility)
 				{
-					title = "Playlist content";
-					title += " (";
-					title += boost::lexical_cast<std::string>(Content.size());
-					title += " item";
-					if (Content.size() == 1)
-						title += ")";
-					else
-						title += "s)";
-					title.resize(Content.getWidth());
+					Content[idx].setBold(in_playlist);
+					Content[idx].value() = std::move(*s);
 				}
-				Content.setTitle(title);
-			});
+				else
+				{
+					auto properties = NC::List::Properties::Selectable;
+					if (in_playlist)
+						properties |= NC::List::Properties::Bold;
+					Content.addItem(std::move(*s), properties);
+				}
+			}
+			if (idx < Content.size())
+				Content.resizeList(idx);
+			std::string wtitle;
+			if (Config.titles_visibility)
+			{
+				wtitle = (boost::format("Playlist content (%1%) %2%")
+					% boost::lexical_cast<std::string>(Content.size())
+					% (Content.size() == 1 ? "item" : "items")
+					).str();
+				wtitle.resize(Content.getWidth());
+			}
+			Content.setTitle(wtitle);
 		}
 		Content.display();
 	}
 	
-	if (isActiveWindow(Content) && Content.reallyEmpty())
+	if (isActiveWindow(Content) && Content.empty())
 	{
 		Content.setHighlightColor(Config.main_highlight_color);
 		Playlists.setHighlightColor(Config.active_column_color);
 		w = &Playlists;
 	}
 	
-	if (Playlists.empty() && Content.reallyEmpty())
+	if (Playlists.empty() && Content.empty())
 	{
 		Content.Window::clear();
 		Content.Window::display();
@@ -214,77 +219,12 @@ void PlaylistEditor::update()
 
 int PlaylistEditor::windowTimeout()
 {
-	if (Content.reallyEmpty())
+	if (Content.empty())
 		return m_window_timeout;
 	else
 		return Screen<WindowType>::windowTimeout();
 }
 
-bool PlaylistEditor::isContentFiltered()
-{
-	if (Content.isFiltered())
-	{
-		Statusbar::print("Function currently unavailable due to filtered playlist content");
-		return true;
-	}
-	return false;
-}
-
-ProxySongList PlaylistEditor::contentProxyList()
-{
-	return ProxySongList(Content, [](NC::Menu<MPD::Song>::Item &item) {
-		return &item.value();
-	});
-}
-
-void PlaylistEditor::AddToPlaylist(bool add_n_play)
-{
-	if (isActiveWindow(Playlists) && !Playlists.empty())
-	{
-		MPD::SongList list;
-		Mpd.GetPlaylistContent(Playlists.current().value(), vectorMoveInserter(list));
-		bool success = addSongsToPlaylist(list.begin(), list.end(), add_n_play, -1);
-		Statusbar::printf("Playlist \"%1%\" loaded%2%",
-			Playlists.current().value(), withErrors(success)
-		);
-	}
-	else if (isActiveWindow(Content) && !Content.empty())
-		addSongToPlaylist(Content.current().value(), add_n_play);
-	
-	if (!add_n_play)
-		w->scroll(NC::Scroll::Down);
-}
-
-void PlaylistEditor::enterPressed()
-{
-	AddToPlaylist(true);
-}
-
-void PlaylistEditor::spacePressed()
-{
-	if (Config.space_selects)
-	{
-		if (isActiveWindow(Playlists))
-		{
-			if (!Playlists.empty())
-			{
-				Playlists.current().setSelected(!Playlists.current().isSelected());
-				Playlists.scroll(NC::Scroll::Down);
-			}
-		}
-		else if (isActiveWindow(Content))
-		{
-			if (!Content.empty())
-			{
-				Content.current().setSelected(!Content.current().isSelected());
-				Content.scroll(NC::Scroll::Down);
-			}
-		}
-	}
-	else
-		AddToPlaylist(false);
-}
-
 void PlaylistEditor::mouseButtonPressed(MEVENT me)
 {
 	if (!Playlists.empty() && Playlists.hasCoords(me.x, me.y))
@@ -300,12 +240,7 @@ void PlaylistEditor::mouseButtonPressed(MEVENT me)
 		{
 			Playlists.Goto(me.y);
 			if (me.bstate & BUTTON3_PRESSED)
-			{
-				size_t pos = Playlists.choice();
-				spacePressed();
-				if (pos < Playlists.size()-1)
-					Playlists.scroll(NC::Scroll::Up);
-			}
+				addItemToPlaylist(false);
 		}
 		else
 			Screen<WindowType>::mouseButtonPressed(me);
@@ -323,15 +258,8 @@ void PlaylistEditor::mouseButtonPressed(MEVENT me)
 		if (size_t(me.y) < Content.size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
 		{
 			Content.Goto(me.y);
-			if (me.bstate & BUTTON1_PRESSED)
-			{
-				size_t pos = Content.choice();
-				spacePressed();
-				if (pos < Content.size()-1)
-					Content.scroll(NC::Scroll::Up);
-			}
-			else
-				enterPressed();
+			bool play = me.bstate & BUTTON3_PRESSED;
+			addItemToPlaylist(play);
 		}
 		else
 			Screen<WindowType>::mouseButtonPressed(me);
@@ -340,139 +268,80 @@ void PlaylistEditor::mouseButtonPressed(MEVENT me)
 
 /***********************************************************************/
 
-bool PlaylistEditor::allowsFiltering()
-{
-	return true;
-}
-
-std::string PlaylistEditor::currentFilter()
-{
-	std::string filter;
-	if (isActiveWindow(Playlists))
-		filter = RegexFilter<std::string>::currentFilter(Playlists);
-	else if (isActiveWindow(Content))
-		filter = RegexFilter<MPD::Song>::currentFilter(Content);
-	return filter;
-}
-
-void PlaylistEditor::applyFilter(const std::string &filter)
-{
-	if (filter.empty())
-	{
-		if (isActiveWindow(Playlists))
-		{
-			Playlists.clearFilter();
-			Playlists.clearFilterResults();
-		}
-		else if (isActiveWindow(Content))
-		{
-			Content.clearFilter();
-			Content.clearFilterResults();
-		}
-		return;
-	}
-	try
-	{
-		if (isActiveWindow(Playlists))
-		{
-			Playlists.showAll();
-			auto rx = RegexFilter<std::string>(
-				boost::regex(filter, Config.regex_type), PlaylistEntryMatcher);
-			Playlists.filter(Playlists.begin(), Playlists.end(), rx);
-		}
-		else if (isActiveWindow(Content))
-		{
-			Content.showAll();
-			auto rx = RegexFilter<MPD::Song>(
-				boost::regex(filter, Config.regex_type), SongEntryMatcher);
-			Content.filter(Content.begin(), Content.end(), rx);
-		}
-	}
-	catch (boost::bad_expression &) { }
-}
-
-/***********************************************************************/
-
 bool PlaylistEditor::allowsSearching()
 {
 	return true;
 }
 
-bool PlaylistEditor::search(const std::string &constraint)
+void PlaylistEditor::setSearchConstraint(const std::string &constraint)
 {
-	if (constraint.empty())
+	if (isActiveWindow(Playlists))
 	{
-		if (isActiveWindow(Playlists))
-			Playlists.clearSearchResults();
-		else if (isActiveWindow(Content))
-			Content.clearSearchResults();
-		return false;
+		m_playlists_search_predicate = Regex::Filter<MPD::Playlist>(
+			Regex::make(constraint, Config.regex_type),
+			PlaylistEntryMatcher
+		);
 	}
-	try
+	else if (isActiveWindow(Content))
 	{
-		bool result = false;
-		if (isActiveWindow(Playlists))
-		{
-			auto rx = RegexFilter<std::string>(
-				boost::regex(constraint, Config.regex_type), PlaylistEntryMatcher);
-			result = Playlists.search(Playlists.begin(), Playlists.end(), rx);
-		}
-		else if (isActiveWindow(Content))
-		{
-			auto rx = RegexFilter<MPD::Song>(
-				boost::regex(constraint, Config.regex_type), SongEntryMatcher);
-			result = Content.search(Content.begin(), Content.end(), rx);
-		}
-		return result;
-	}
-	catch (boost::bad_expression &)
-	{
-		return false;
+		m_content_search_predicate = Regex::Filter<MPD::Song>(
+			Regex::make(constraint, Config.regex_type),
+			SongEntryMatcher
+		);
 	}
 }
 
-void PlaylistEditor::nextFound(bool wrap)
+void PlaylistEditor::clearConstraint()
 {
 	if (isActiveWindow(Playlists))
-		Playlists.nextFound(wrap);
+		m_playlists_search_predicate.clear();
 	else if (isActiveWindow(Content))
-		Content.nextFound(wrap);
+		m_content_search_predicate.clear();
 }
 
-void PlaylistEditor::prevFound(bool wrap)
+bool PlaylistEditor::find(SearchDirection direction, bool wrap, bool skip_current)
 {
+	bool result = false;
 	if (isActiveWindow(Playlists))
-		Playlists.prevFound(wrap);
+		result = search(Playlists, m_playlists_search_predicate, direction, wrap, skip_current);
 	else if (isActiveWindow(Content))
-		Content.prevFound(wrap);
+		result = search(Content, m_content_search_predicate, direction, wrap, skip_current);
+	return result;
 }
 
 /***********************************************************************/
 
-ProxySongList PlaylistEditor::proxySongList()
-{
-	auto ptr = ProxySongList();
-	if (isActiveWindow(Content))
-		ptr = contentProxyList();
-	return ptr;
-}
-
-bool PlaylistEditor::allowsSelection()
-{
-	return true;
-}
-
-void PlaylistEditor::reverseSelection()
+bool PlaylistEditor::itemAvailable()
 {
 	if (isActiveWindow(Playlists))
-		reverseSelectionHelper(Playlists.begin(), Playlists.end());
-	else if (isActiveWindow(Content))
-		reverseSelectionHelper(Content.begin(), Content.end());
+		return !Playlists.empty();
+	if (isActiveWindow(Content))
+		return !Content.empty();
+	return false;
 }
 
-MPD::SongList PlaylistEditor::getSelectedSongs()
+bool PlaylistEditor::addItemToPlaylist(bool play)
 {
-	MPD::SongList result;
+	bool result = false;
+	if (isActiveWindow(Playlists))
+	{
+		std::vector<MPD::Song> list(
+			std::make_move_iterator(Mpd.GetPlaylistContent(Playlists.current()->value().path())),
+			std::make_move_iterator(MPD::SongIterator())
+		);
+		result = addSongsToPlaylist(list.begin(), list.end(), play, -1);
+		Statusbar::printf("Playlist \"%1%\" loaded%2%",
+			Playlists.current()->value().path(), withErrors(result)
+		);
+	}
+	else if (isActiveWindow(Content))
+		result = addSongToPlaylist(Content.current()->value(), play);
+	return result;
+}
+
+std::vector<MPD::Song> PlaylistEditor::getSelectedSongs()
+{
+	std::vector<MPD::Song> result;
 	if (isActiveWindow(Playlists))
 	{
 		bool any_selected = false;
@@ -481,22 +350,25 @@ MPD::SongList PlaylistEditor::getSelectedSongs()
 			if (e.isSelected())
 			{
 				any_selected = true;
-				Mpd.GetPlaylistContent(e.value(), vectorMoveInserter(result));
+				std::copy(
+					std::make_move_iterator(Mpd.GetPlaylistContent(e.value().path())),
+					std::make_move_iterator(MPD::SongIterator()),
+					std::back_inserter(result)
+				);
 			}
 		}
 		// if no item is selected, add songs from right column
 		if (!any_selected && !Playlists.empty())
-			Mpd.GetPlaylistContent(Playlists.current().value(), vectorMoveInserter(result));
+		{
+			std::copy(
+				std::make_move_iterator(Mpd.GetPlaylistContent(Playlists.current()->value().path())),
+				std::make_move_iterator(MPD::SongIterator()),
+				std::back_inserter(result)
+			);
+		}
 	}
 	else if (isActiveWindow(Content))
-	{
-		for (auto &e : Content)
-			if (e.isSelected())
-				result.push_back(e.value());
-		// if no item is selected, add current one
-		if (result.empty() && !Content.empty())
-			result.push_back(Content.current().value());
-	}
+		result = Content.getSelectedSongs();
 	return result;
 }
 
@@ -506,7 +378,7 @@ bool PlaylistEditor::previousColumnAvailable()
 {
 	if (isActiveWindow(Content))
 	{
-		if (!Playlists.reallyEmpty())
+		if (!Playlists.empty())
 			return true;
 	}
 	return false;
@@ -527,7 +399,7 @@ bool PlaylistEditor::nextColumnAvailable()
 {
 	if (isActiveWindow(Playlists))
 	{
-		if (!Content.reallyEmpty())
+		if (!Content.empty())
 			return true;
 	}
 	return false;
@@ -551,22 +423,20 @@ void PlaylistEditor::updateTimer()
 	m_timer = Global::Timer;
 }
 
-void PlaylistEditor::Locate(const std::string &name)
+void PlaylistEditor::Locate(const MPD::Playlist &playlist)
 {
 	update();
-	for (size_t i = 0; i < Playlists.size(); ++i)
+	auto begin = Playlists.beginV(), end = Playlists.endV();
+	auto it = std::find(begin, end, playlist);
+	if (it != end)
 	{
-		if (name == Playlists[i].value())
-		{
-			Playlists.highlight(i);
-			Content.clear();
-			break;
-		}
+		Playlists.highlight(it-begin);
+		Content.clear();
+		switchTo();
 	}
-	switchTo();
 }
 
-namespace {//
+namespace {
 
 std::string SongToString(const MPD::Song &s)
 {
@@ -574,23 +444,23 @@ std::string SongToString(const MPD::Song &s)
 	switch (Config.playlist_display_mode)
 	{
 		case DisplayMode::Classic:
-			result = s.toString(Config.song_list_format_dollar_free, Config.tags_separator);
+			result = Format::stringify<char>(Config.song_list_format, &s);
 			break;
 		case DisplayMode::Columns:
-			result = s.toString(Config.song_in_columns_to_string_format, Config.tags_separator);
+			result = Format::stringify<char>(Config.song_columns_mode_format, &s);
 			break;
 	}
 	return result;
 }
 
-bool PlaylistEntryMatcher(const boost::regex &rx, const std::string &playlist)
+bool PlaylistEntryMatcher(const Regex::Regex &rx, const MPD::Playlist &playlist)
 {
-	return boost::regex_search(playlist, rx);
+	return Regex::search(playlist.path(), rx);
 }
 
-bool SongEntryMatcher(const boost::regex &rx, const MPD::Song &s)
+bool SongEntryMatcher(const Regex::Regex &rx, const MPD::Song &s)
 {
-	return boost::regex_search(SongToString(s), rx);
+	return Regex::search(SongToString(s), rx);
 }
 
 }
