@@ -19,18 +19,19 @@
  ***************************************************************************/
 
 #include <array>
-#include <boost/bind.hpp>
+#include <boost/range/detail/any_iterator.hpp>
 #include <iomanip>
 
 #include "display.h"
 #include "global.h"
 #include "helpers.h"
+#include "menu_impl.h"
 #include "playlist.h"
-#include "regex_filter.h"
 #include "search_engine.h"
 #include "settings.h"
 #include "status.h"
 #include "statusbar.h"
+#include "helpers/song_iterator_maker.h"
 #include "utility/comparators.h"
 #include "title.h"
 #include "screen_switcher.h"
@@ -38,9 +39,11 @@
 using Global::MainHeight;
 using Global::MainStartY;
 
+namespace ph = std::placeholders;
+
 SearchEngine *mySearcher;
 
-namespace {//
+namespace {
 
 /*const std::array<const std::string, 11> constraintsNames = {{
 	"Any",
@@ -62,7 +65,7 @@ const std::array<const char *, 3> searchModes = {{
 	"Match only if both values are the same"
 }};
 
-namespace pos {//
+namespace pos {
 	const size_t searchIn = constraintsNames.size()-1+1+1; // separated
 	const size_t searchMode = searchIn+1;
 	const size_t search = searchMode+1+1; // separated
@@ -70,10 +73,65 @@ namespace pos {//
 }*/
 
 std::string SEItemToString(const SEItem &ei);
-bool SEItemEntryMatcher(const boost::regex &rx, const NC::Menu<SEItem>::Item &item, bool filter);
+bool SEItemEntryMatcher(const Regex::Regex &rx, const NC::Menu<SEItem>::Item &item, bool filter);
+
+template <bool Const>
+struct SongExtractor
+{
+	typedef SongExtractor type;
+
+	typedef typename NC::Menu<SEItem>::Item MenuItem;
+	typedef typename std::conditional<Const, const MenuItem, MenuItem>::type Item;
+	typedef typename std::conditional<Const, const MPD::Song, MPD::Song>::type Song;
+
+	Song *operator()(Item &item) const
+	{
+		Song *ptr = nullptr;
+		if (!item.isSeparator() && item.value().isSong())
+			ptr = &item.value().song();
+		return ptr;
+	}
+};
 
 }
 
+SongIterator SearchEngineWindow::currentS()
+{
+	return makeSongIterator_<SEItem>(current(), SongExtractor<false>());
+}
+
+ConstSongIterator SearchEngineWindow::currentS() const
+{
+	return makeConstSongIterator_<SEItem>(current(), SongExtractor<true>());
+}
+
+SongIterator SearchEngineWindow::beginS()
+{
+	return makeSongIterator_<SEItem>(begin(), SongExtractor<false>());
+}
+
+ConstSongIterator SearchEngineWindow::beginS() const
+{
+	return makeConstSongIterator_<SEItem>(begin(), SongExtractor<true>());
+}
+
+SongIterator SearchEngineWindow::endS()
+{
+	return makeSongIterator_<SEItem>(end(), SongExtractor<false>());
+}
+
+ConstSongIterator SearchEngineWindow::endS() const
+{
+	return makeConstSongIterator_<SEItem>(end(), SongExtractor<true>());
+}
+
+std::vector<MPD::Song> SearchEngineWindow::getSelectedSongs()
+{
+	return {}; // TODO
+}
+
+/**********************************************************************/
+
 const char *SearchEngine::ConstraintsNames[] =
 {
 	"Any",
@@ -102,12 +160,12 @@ size_t SearchEngine::ResetButton = 16;
 size_t SearchEngine::SearchButton = 15;
 
 SearchEngine::SearchEngine()
-: Screen(NC::Menu<SEItem>(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border::None))
+: Screen(NC::Menu<SEItem>(0, MainStartY, COLS, MainHeight, "", Config.main_color, NC::Border()))
 {
 	w.setHighlightColor(Config.main_highlight_color);
 	w.cyclicScrolling(Config.use_cyclic_scrolling);
 	w.centeredCursor(Config.centered_cursor);
-	w.setItemDisplayer(boost::bind(Display::SEItems, _1, proxySongList()));
+	w.setItemDisplayer(std::bind(Display::SEItems, ph::_1, std::cref(w)));
 	w.setSelectedPrefix(Config.selected_item_prefix);
 	w.setSelectedSuffix(Config.selected_item_suffix);
 	SearchMode = &SearchModes[Config.search_engine_default_search_mode];
@@ -138,7 +196,7 @@ void SearchEngine::switchTo()
 	SwitchTo::execute(this);
 	if (w.empty())
 		Prepare();
-	markSongsInPlaylist(proxySongList());
+	markSongsInPlaylist(w);
 	drawHeader();
 }
 
@@ -147,53 +205,105 @@ std::wstring SearchEngine::title()
 	return L"Search engine";
 }
 
-void SearchEngine::enterPressed()
+void SearchEngine::mouseButtonPressed(MEVENT me)
+{
+	if (w.empty() || !w.hasCoords(me.x, me.y) || size_t(me.y) >= w.size())
+		return;
+	if (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED))
+	{
+		if (!w.Goto(me.y))
+			return;
+		w.refresh();
+		if ((me.bstate & BUTTON3_PRESSED)
+		    && w.choice() < StaticOptions)
+			runAction();
+		else if (w.choice() >= StaticOptions)
+		{
+			bool play = me.bstate & BUTTON3_PRESSED;
+			addItemToPlaylist(play);
+		}
+	}
+	else
+		Screen<WindowType>::mouseButtonPressed(me);
+}
+
+/***********************************************************************/
+
+bool SearchEngine::allowsSearching()
+{
+	return w.rbegin()->value().isSong();
+}
+
+void SearchEngine::setSearchConstraint(const std::string &constraint)
+{
+	m_search_predicate = Regex::ItemFilter<SEItem>(
+		Regex::make(constraint, Config.regex_type),
+		std::bind(SEItemEntryMatcher, ph::_1, ph::_2, false)
+	);
+}
+
+void SearchEngine::clearConstraint()
+{
+	m_search_predicate.clear();
+}
+
+bool SearchEngine::find(SearchDirection direction, bool wrap, bool skip_current)
+{
+	return search(w, m_search_predicate, direction, wrap, skip_current);
+}
+
+/***********************************************************************/
+
+bool SearchEngine::actionRunnable()
+{
+	return !w.empty() && !w.current()->value().isSong();
+}
+
+void SearchEngine::runAction()
 {
 	size_t option = w.choice();
 	if (option > ConstraintsNumber && option < SearchButton)
-		w.current().value().buffer().clear();
-	if (option < SearchButton)
-		Statusbar::lock();
-	
+		w.current()->value().buffer().clear();
+
 	if (option < ConstraintsNumber)
 	{
+		Statusbar::ScopedLock slock;
 		std::string constraint = ConstraintsNames[option];
 		Statusbar::put() << NC::Format::Bold << constraint << NC::Format::NoBold << ": ";
-		itsConstraints[option] = Global::wFooter->getString(itsConstraints[option]);
-		w.current().value().buffer().clear();
+		itsConstraints[option] = Global::wFooter->prompt(itsConstraints[option]);
+		w.current()->value().buffer().clear();
 		constraint.resize(13, ' ');
-		w.current().value().buffer() << NC::Format::Bold << constraint << NC::Format::NoBold << ": ";
-		ShowTag(w.current().value().buffer(), itsConstraints[option]);
+		w.current()->value().buffer() << NC::Format::Bold << constraint << NC::Format::NoBold << ": ";
+		ShowTag(w.current()->value().buffer(), itsConstraints[option]);
 	}
 	else if (option == ConstraintsNumber+1)
 	{
 		Config.search_in_db = !Config.search_in_db;
-		w.current().value().buffer() << NC::Format::Bold << "Search in:" << NC::Format::NoBold << ' ' << (Config.search_in_db ? "Database" : "Current playlist");
+		w.current()->value().buffer() << NC::Format::Bold << "Search in:" << NC::Format::NoBold << ' ' << (Config.search_in_db ? "Database" : "Current playlist");
 	}
 	else if (option == ConstraintsNumber+2)
 	{
 		if (!*++SearchMode)
 			SearchMode = &SearchModes[0];
-		w.current().value().buffer() << NC::Format::Bold << "Search mode:" << NC::Format::NoBold << ' ' << *SearchMode;
+		w.current()->value().buffer() << NC::Format::Bold << "Search mode:" << NC::Format::NoBold << ' ' << *SearchMode;
 	}
 	else if (option == SearchButton)
 	{
-		w.showAll();
 		Statusbar::print("Searching...");
 		if (w.size() > StaticOptions)
 			Prepare();
 		Search();
-		if (w.back().value().isSong())
+		if (w.rbegin()->value().isSong())
 		{
 			if (Config.search_engine_display_mode == DisplayMode::Columns)
 				w.setTitle(Config.titles_visibility ? Display::Columns(w.getWidth()) : "");
 			size_t found = w.size()-SearchEngine::StaticOptions;
 			found += 3; // don't count options inserted below
 			w.insertSeparator(ResetButton+1);
-			w.insertItem(ResetButton+2, SEItem(), 1, 1);
+			w.insertItem(ResetButton+2, SEItem(), NC::List::Properties::Bold | NC::List::Properties::Inactive);
 			w.at(ResetButton+2).value().mkBuffer() << Config.color1 << "Search results: " << Config.color2 << "Found " << found << (found > 1 ? " songs" : " song") << NC::Color::Default;
 			w.insertSeparator(ResetButton+3);
-			markSongsInPlaylist(proxySongList());
+			markSongsInPlaylist(w);
 			Statusbar::print("Searching finished");
 			if (Config.block_search_constraints_change)
 				for (size_t i = 0; i < StaticOptions-4; ++i)
@@ -209,149 +319,24 @@ void SearchEngine::enterPressed()
 		reset();
 	}
 	else
-		addSongToPlaylist(w.current().value().song(), true);
-	
-	if (option < SearchButton)
-		Statusbar::unlock();
-}
-
-void SearchEngine::spacePressed()
-{
-	if (!w.current().value().isSong())
-		return;
-	
-	if (Config.space_selects)
-	{
-		w.current().setSelected(!w.current().isSelected());
-		w.scroll(NC::Scroll::Down);
-		return;
-	}
-	
-	addSongToPlaylist(w.current().value().song(), false);
-	w.scroll(NC::Scroll::Down);
-}
-
-void SearchEngine::mouseButtonPressed(MEVENT me)
-{
-	if (w.empty() || !w.hasCoords(me.x, me.y) || size_t(me.y) >= w.size())
-		return;
-	if (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED))
-	{
-		if (!w.Goto(me.y))
-			return;
-		w.refresh();
-		if ((me.bstate & BUTTON3_PRESSED || w.choice() > ConstraintsNumber) && w.choice() < StaticOptions)
-			enterPressed();
-		else if (w.choice() >= StaticOptions)
-		{
-			if (me.bstate & BUTTON1_PRESSED)
-			{
-				size_t pos = w.choice();
-				spacePressed();
-				if (pos < w.size()-1)
-					w.scroll(NC::Scroll::Up);
-			}
-			else
-				enterPressed();
-		}
-	}
-	else
-		Screen<WindowType>::mouseButtonPressed(me);
+		addSongToPlaylist(w.current()->value().song(), true);
 }
 
 /***********************************************************************/
 
-bool SearchEngine::allowsFiltering()
+bool SearchEngine::itemAvailable()
 {
-	return w.back().value().isSong();
+	return !w.empty() && w.current()->value().isSong();
 }
 
-std::string SearchEngine::currentFilter()
+bool SearchEngine::addItemToPlaylist(bool play)
 {
-	return RegexItemFilter<SEItem>::currentFilter(w);
+	return addSongToPlaylist(w.current()->value().song(), play);
 }
 
-void SearchEngine::applyFilter(const std::string &filter)
+std::vector<MPD::Song> SearchEngine::getSelectedSongs()
 {
-	if (filter.empty())
-	{
-		w.clearFilter();
-		w.clearFilterResults();
-		return;
-	}
-	try
-	{
-		w.showAll();
-		auto fun = boost::bind(SEItemEntryMatcher, _1, _2, true);
-		auto rx = RegexItemFilter<SEItem>(
-			boost::regex(filter, Config.regex_type), fun);
-		w.filter(w.begin(), w.end(), rx);
-	}
-	catch (boost::bad_expression &) { }
-}
-
-/***********************************************************************/
-
-bool SearchEngine::allowsSearching()
-{
-	return w.back().value().isSong();
-}
-
-bool SearchEngine::search(const std::string &constraint)
-{
-	if (constraint.empty())
-	{
-		w.clearSearchResults();
-		return false;
-	}
-	try
-	{
-		auto fun = boost::bind(SEItemEntryMatcher, _1, _2, false);
-		auto rx = RegexItemFilter<SEItem>(
-			boost::regex(constraint, Config.regex_type), fun);
-		return w.search(w.begin(), w.end(), rx);
-	}
-	catch (boost::bad_expression &)
-	{
-		return false;
-	}
-}
-
-void SearchEngine::nextFound(bool wrap)
-{
-	w.nextFound(wrap);
-}
-
-void SearchEngine::prevFound(bool wrap)
-{
-	w.prevFound(wrap);
-}
-
-/***********************************************************************/
-
-ProxySongList SearchEngine::proxySongList()
-{
-	return ProxySongList(w, [](NC::Menu<SEItem>::Item &item) -> MPD::Song * {
-		MPD::Song *ptr = 0;
-		if (!item.isSeparator() && item.value().isSong())
-			ptr = &item.value().song();
-		return ptr;
-	});
-}
-
-bool SearchEngine::allowsSelection()
-{
-	return w.current().value().isSong();
-}
-
-void SearchEngine::reverseSelection()
-{
-	reverseSelectionHelper(w.begin()+std::min(StaticOptions, w.size()), w.end());
-}
-
-MPD::SongList SearchEngine::getSelectedSongs()
-{
-	MPD::SongList result;
+	std::vector<MPD::Song> result;
 	for (auto it = w.begin(); it != w.end(); ++it)
 	{
 		if (it->isSelected())
@@ -363,8 +348,8 @@ MPD::SongList SearchEngine::getSelectedSongs()
 	// if no item is selected, add current one
 	if (result.empty() && !w.empty())
 	{
-		assert(w.current().value().isSong());
-		result.push_back(w.current().value().song());
+		assert(w.current()->value().isSong());
+		result.push_back(w.current()->value().song());
 	}
 	return result;
 }
@@ -376,6 +361,9 @@ void SearchEngine::Prepare()
 	w.setTitle("");
 	w.clear();
 	w.resizeList(StaticOptions-3);
+
+	for (auto &item : w)
+		item.setSelectable(false);
 	
 	w.at(ConstraintsNumber).setSeparator(true);
 	w.at(SearchButton-1).setSeparator(true);
@@ -443,183 +431,128 @@ void SearchEngine::Search()
 			Mpd.AddSearch(MPD_TAG_DATE, itsConstraints[9]);
 		if (!itsConstraints[10].empty())
 			Mpd.AddSearch(MPD_TAG_COMMENT, itsConstraints[10]);
-		Mpd.CommitSearchSongs([this](MPD::Song s) {
-			w.addItem(s);
-		});
+		for (MPD::SongIterator s = Mpd.CommitSearchSongs(), end; s != end; ++s)
+			w.addItem(std::move(*s));
 		return;
 	}
-	
-	MPD::SongList list;
-	if (Config.search_in_db)
-		Mpd.GetDirectoryRecursive("/", vectorMoveInserter(list));
-	else
-		list.insert(list.end(), myPlaylist->main().beginV(), myPlaylist->main().endV());
-	
-	bool any_found = 1;
-	bool found = 1;
-	
-	LocaleStringComparison cmp(std::locale(), Config.ignore_leading_the);
-	for (auto it = list.begin(); it != list.end(); ++it)
+
+	Regex::Regex rx[ConstraintsNumber];
+	if (SearchMode != &SearchModes[2]) // match to pattern
 	{
+		for (size_t i = 0; i < ConstraintsNumber; ++i)
+		{
+			if (!itsConstraints[i].empty())
+			{
+				try
+				{
+					rx[i] = Regex::make(itsConstraints[i], Config.regex_type);
+				}
+				catch (boost::bad_expression &) { }
+			}
+		}
+	}
+
+	typedef boost::range_detail::any_iterator<
+		const MPD::Song,
+		boost::single_pass_traversal_tag,
+		const MPD::Song &,
+		std::ptrdiff_t
+	> input_song_iterator;
+	input_song_iterator s, end;
+	if (Config.search_in_db)
+	{
+		s = input_song_iterator(getDatabaseIterator(Mpd));
+		end = input_song_iterator(MPD::SongIterator());
+	}
+	else
+	{
+		s = input_song_iterator(myPlaylist->main().beginV());
+		end = input_song_iterator(myPlaylist->main().endV());
+	}
+
+	LocaleStringComparison cmp(std::locale(), Config.ignore_leading_the);
+	for (; s != end; ++s)
+	{
+		bool any_found = true, found = true;
+
 		if (SearchMode != &SearchModes[2]) // match to pattern
 		{
-			boost::regex rx;
-			if (!itsConstraints[0].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[0], Config.regex_type);
-					any_found =
-					   boost::regex_search(it->getArtist(), rx)
-					|| boost::regex_search(it->getAlbumArtist(), rx)
-					|| boost::regex_search(it->getTitle(), rx)
-					|| boost::regex_search(it->getAlbum(), rx)
-					|| boost::regex_search(it->getName(), rx)
-					|| boost::regex_search(it->getComposer(), rx)
-					|| boost::regex_search(it->getPerformer(), rx)
-					|| boost::regex_search(it->getGenre(), rx)
-					|| boost::regex_search(it->getDate(), rx)
-					|| boost::regex_search(it->getComment(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			
-			if (found && !itsConstraints[1].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[1], Config.regex_type);
-					found = boost::regex_search(it->getArtist(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[2].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[2], Config.regex_type);
-					found = boost::regex_search(it->getAlbumArtist(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[3].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[3], Config.regex_type);
-					found = boost::regex_search(it->getTitle(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[4].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[4], Config.regex_type);
-					found = boost::regex_search(it->getAlbum(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[5].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[5], Config.regex_type);
-					found = boost::regex_search(it->getName(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[6].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[6], Config.regex_type);
-					found = boost::regex_search(it->getComposer(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[7].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[7], Config.regex_type);
-					found = boost::regex_search(it->getPerformer(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && itsConstraints[8].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[8], Config.regex_type);
-					found = boost::regex_search(it->getGenre(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[9].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[9], Config.regex_type);
-					found = boost::regex_search(it->getDate(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
-			if (found && !itsConstraints[10].empty())
-			{
-				try
-				{
-					rx.assign(itsConstraints[10], Config.regex_type);
-					found = boost::regex_search(it->getComment(), rx);
-				}
-				catch (boost::bad_expression &) { }
-			}
+			if (!rx[0].empty())
+				any_found =
+				   Regex::search(s->getArtist(), rx[0])
+				|| Regex::search(s->getAlbumArtist(), rx[0])
+				|| Regex::search(s->getTitle(), rx[0])
+				|| Regex::search(s->getAlbum(), rx[0])
+				|| Regex::search(s->getName(), rx[0])
+				|| Regex::search(s->getComposer(), rx[0])
+				|| Regex::search(s->getPerformer(), rx[0])
+				|| Regex::search(s->getGenre(), rx[0])
+				|| Regex::search(s->getDate(), rx[0])
+				|| Regex::search(s->getComment(), rx[0]);
+			if (found && !rx[1].empty())
+				found = Regex::search(s->getArtist(), rx[1]);
+			if (found && !rx[2].empty())
+				found = Regex::search(s->getAlbumArtist(), rx[2]);
+			if (found && !rx[3].empty())
+				found = Regex::search(s->getTitle(), rx[3]);
+			if (found && !rx[4].empty())
+				found = Regex::search(s->getAlbum(), rx[4]);
+			if (found && !rx[5].empty())
+				found = Regex::search(s->getName(), rx[5]);
+			if (found && !rx[6].empty())
+				found = Regex::search(s->getComposer(), rx[6]);
+			if (found && !rx[7].empty())
+				found = Regex::search(s->getPerformer(), rx[7]);
+			if (found && !rx[8].empty())
+				found = Regex::search(s->getGenre(), rx[8]);
+			if (found && !rx[9].empty())
+				found = Regex::search(s->getDate(), rx[9]);
+			if (found && !rx[10].empty())
+				found = Regex::search(s->getComment(), rx[10]);
 		}
 		else // match only if values are equal
 		{
 			if (!itsConstraints[0].empty())
 				any_found =
-				   !cmp(it->getArtist(), itsConstraints[0])
-				|| !cmp(it->getAlbumArtist(), itsConstraints[0])
-				|| !cmp(it->getTitle(), itsConstraints[0])
-				|| !cmp(it->getAlbum(), itsConstraints[0])
-				|| !cmp(it->getName(), itsConstraints[0])
-				|| !cmp(it->getComposer(), itsConstraints[0])
-				|| !cmp(it->getPerformer(), itsConstraints[0])
-				|| !cmp(it->getGenre(), itsConstraints[0])
-				|| !cmp(it->getDate(), itsConstraints[0])
-				|| !cmp(it->getComment(), itsConstraints[0]);
+				   !cmp(s->getArtist(), itsConstraints[0])
+				|| !cmp(s->getAlbumArtist(), itsConstraints[0])
+				|| !cmp(s->getTitle(), itsConstraints[0])
+				|| !cmp(s->getAlbum(), itsConstraints[0])
+				|| !cmp(s->getName(), itsConstraints[0])
+				|| !cmp(s->getComposer(), itsConstraints[0])
+				|| !cmp(s->getPerformer(), itsConstraints[0])
+				|| !cmp(s->getGenre(), itsConstraints[0])
+				|| !cmp(s->getDate(), itsConstraints[0])
+				|| !cmp(s->getComment(), itsConstraints[0]);
 			
 			if (found && !itsConstraints[1].empty())
-				found = !cmp(it->getArtist(), itsConstraints[1]);
+				found = !cmp(s->getArtist(), itsConstraints[1]);
 			if (found && !itsConstraints[2].empty())
-				found = !cmp(it->getAlbumArtist(), itsConstraints[2]);
+				found = !cmp(s->getAlbumArtist(), itsConstraints[2]);
 			if (found && !itsConstraints[3].empty())
-				found = !cmp(it->getTitle(), itsConstraints[3]);
+				found = !cmp(s->getTitle(), itsConstraints[3]);
 			if (found && !itsConstraints[4].empty())
-				found = !cmp(it->getAlbum(), itsConstraints[4]);
+				found = !cmp(s->getAlbum(), itsConstraints[4]);
 			if (found && !itsConstraints[5].empty())
-				found = !cmp(it->getName(), itsConstraints[5]);
+				found = !cmp(s->getName(), itsConstraints[5]);
 			if (found && !itsConstraints[6].empty())
-				found = !cmp(it->getComposer(), itsConstraints[6]);
+				found = !cmp(s->getComposer(), itsConstraints[6]);
 			if (found && !itsConstraints[7].empty())
-				found = !cmp(it->getPerformer(), itsConstraints[7]);
+				found = !cmp(s->getPerformer(), itsConstraints[7]);
 			if (found && !itsConstraints[8].empty())
-				found = !cmp(it->getGenre(), itsConstraints[8]);
+				found = !cmp(s->getGenre(), itsConstraints[8]);
 			if (found && !itsConstraints[9].empty())
-				found = !cmp(it->getDate(), itsConstraints[9]);
+				found = !cmp(s->getDate(), itsConstraints[9]);
 			if (found && !itsConstraints[10].empty())
-				found = !cmp(it->getComment(), itsConstraints[10]);
+				found = !cmp(s->getComment(), itsConstraints[10]);
 		}
 		
-		if (found && any_found)
-			w.addItem(*it);
-		found = 1;
-		any_found = 1;
+		if (any_found && found)
+			w.addItem(*s);
 	}
 }
 
-namespace {//
+namespace {
 
 std::string SEItemToString(const SEItem &ei)
 {
@@ -629,10 +562,10 @@ std::string SEItemToString(const SEItem &ei)
 		switch (Config.search_engine_display_mode)
 		{
 			case DisplayMode::Classic:
-				result = ei.song().toString(Config.song_list_format_dollar_free, Config.tags_separator);
+				result = Format::stringify<char>(Config.song_list_format, &ei.song());
 				break;
 			case DisplayMode::Columns:
-				result = ei.song().toString(Config.song_in_columns_to_string_format, Config.tags_separator);
+				result = Format::stringify<char>(Config.song_columns_mode_format, &ei.song());
 				break;
 		}
 	}
@@ -641,11 +574,11 @@ std::string SEItemToString(const SEItem &ei)
 	return result;
 }
 
-bool SEItemEntryMatcher(const boost::regex &rx, const NC::Menu<SEItem>::Item &item, bool filter)
+bool SEItemEntryMatcher(const Regex::Regex &rx, const NC::Menu<SEItem>::Item &item, bool filter)
 {
 	if (item.isSeparator() || !item.value().isSong())
 		return filter;
-	return boost::regex_search(SEItemToString(item.value()), rx);
+	return Regex::search(SEItemToString(item.value()), rx);
 }
 
 }
