@@ -22,19 +22,21 @@
 #define NCMPCPP_SETTINGS_H
 
 #include <boost/date_time/posix_time/posix_time_types.hpp>
+#include <boost/optional.hpp>
 #include <boost/regex.hpp>
 #include <cassert>
 #include <vector>
 #include <mpd/client.h>
 
 #include "enums.h"
+#include "format.h"
 #include "screen_type.h"
 #include "strbuffer.h"
 
 struct Column
 {
 	Column() : stretch_limit(-1), right_alignment(0), display_empty_tag(1) { }
-	
+
 	std::wstring name;
 	std::string type;
 	int width;
@@ -50,35 +52,34 @@ struct Configuration
 	Configuration()
 	: playlist_disable_highlight_delay(0), visualizer_sync_interval(0)
 	{ }
-	
-	bool read(const std::string &config_path);
-	
+
+	bool read(const std::vector<std::string> &config_paths, bool ignore_errors);
+
 	std::string ncmpcpp_directory;
 	std::string lyrics_directory;
-	
+
 	std::string mpd_music_dir;
 	std::string visualizer_fifo_path;
 	std::string visualizer_output_name;
 	std::string empty_tag;
-	std::string tags_separator;
-	std::string song_list_format;
-	std::string song_list_format_dollar_free;
-	std::string song_status_format;
-	std::string song_status_format_no_colors;
-	std::string song_window_title_format;
-	std::string song_library_format;
-	std::string tag_editor_album_format;
-	std::string song_in_columns_to_string_format;
-	std::string browser_sort_format;
+
+	Format::AST<char> song_list_format;
+	Format::AST<char> song_window_title_format;
+	Format::AST<char> song_library_format;
+	Format::AST<char> song_columns_mode_format;
+	Format::AST<char> browser_sort_format;
+	Format::AST<char> song_status_format;
+	Format::AST<wchar_t> song_status_wformat;
+	Format::AST<wchar_t> new_header_first_line;
+	Format::AST<wchar_t> new_header_second_line;
+
 	std::string external_editor;
 	std::string system_encoding;
 	std::string execute_on_song_change;
-	std::string new_header_first_line;
-	std::string new_header_second_line;
 	std::string lastfm_preferred_language;
 	std::wstring progressbar;
 	std::wstring visualizer_chars;
-	
+
 	std::string pattern;
 
 	std::vector<Column> columns;
@@ -94,7 +95,7 @@ struct Configuration
 	NC::Buffer now_playing_prefix;
 	NC::Buffer now_playing_suffix;
 	NC::Buffer modified_item_prefix;
-	
+
 	NC::Color color1;
 	NC::Color color2;
 	NC::Color empty_tags_color;
@@ -109,18 +110,21 @@ struct Configuration
 	NC::Color statusbar_color;
 	NC::Color alternative_ui_separator_color;
 	NC::Color active_column_color;
-	NC::Color visualizer_color;
-	
+
+	std::vector<NC::Color> visualizer_colors;
+	VisualizerType visualizer_type;
+
 	NC::Border window_border;
 	NC::Border active_window_border;
-	
+
 	Design design;
 
 	SpaceAddMode space_add_mode;
 
 	mpd_tag_type media_lib_primary_tag;
-	
+
 	bool colors_enabled;
+	bool playlist_show_mpd_host;
 	bool playlist_show_remaining_time;
 	bool playlist_shorten_total_times;
 	bool playlist_separate_albums;
@@ -133,7 +137,6 @@ struct Configuration
 	bool screen_switcher_previous;
 	bool autocenter_mode;
 	bool wrapped_search;
-	bool space_selects;
 	bool incremental_seeking;
 	bool now_playing_lyrics;
 	bool fetch_lyrics_in_background;
@@ -151,7 +154,6 @@ struct Configuration
 	bool ask_before_clearing_playlists;
 	bool mouse_support;
 	bool mouse_list_scroll_whole_page;
-	bool visualizer_use_wave;
 	bool visualizer_in_stereo;
 	bool data_fetching_delay;
 	bool media_library_sort_by_mtime;
@@ -162,7 +164,8 @@ struct Configuration
 	bool ask_for_locked_screen_width_part;
 	bool allow_for_physical_item_deletion;
 	bool progressbar_boldness;
-	
+	bool startup_slave_screen_focus;
+
 	unsigned mpd_connection_timeout;
 	unsigned crossfade_time;
 	unsigned seek_time;
@@ -173,21 +176,22 @@ struct Configuration
 	unsigned search_engine_default_search_mode;
 
 	boost::regex::flag_type regex_type;
-	
+
 	boost::posix_time::seconds playlist_disable_highlight_delay;
 	boost::posix_time::seconds visualizer_sync_interval;
-	
+
 	double visualizer_sample_multiplier;
 	double locked_screen_width_part;
-	
+
 	size_t selected_item_prefix_length;
 	size_t selected_item_suffix_length;
 	size_t now_playing_prefix_length;
 	size_t now_playing_suffix_length;
-	
+
 	ScreenType startup_screen_type;
+	boost::optional<ScreenType> startup_slave_screen_type;
 	std::list<ScreenType> screen_sequence;
-	
+
 	SortMode browser_sort_mode;
 };
 
