@@ -22,7 +22,6 @@
 
 #ifdef HAVE_TAGLIB_H
 
-#include <boost/bind.hpp>
 #include <boost/locale/conversion.hpp>
 #include <algorithm>
 #include <fstream>
@@ -33,9 +32,12 @@
 #include "display.h"
 #include "global.h"
 #include "helpers.h"
+#include "menu_impl.h"
 #include "playlist.h"
 #include "song_info.h"
 #include "statusbar.h"
+#include "helpers/song_iterator_maker.h"
+#include "utility/functional.h"
 #include "utility/comparators.h"
 #include "title.h"
 #include "tags.h"
@@ -45,9 +47,11 @@ using Global::myScreen;
 using Global::MainHeight;
 using Global::MainStartY;
 
+namespace ph = std::placeholders;
+
 TagEditor *myTagEditor;
 
-namespace {//
+namespace {
 
 size_t LeftColumnWidth;
 size_t LeftColumnStartX;
@@ -80,17 +84,69 @@ std::string GenerateFilename(const MPD::MutableSong &s, const std::string &patte
 std::string ParseFilename(MPD::MutableSong &s, std::string mask, bool preview);
 
 std::string SongToString(const MPD::MutableSong &s);
-bool DirEntryMatcher(const boost::regex &rx, const std::pair<std::string, std::string> &dir, bool filter);
-bool SongEntryMatcher(const boost::regex &rx, const MPD::MutableSong &s);
+bool DirEntryMatcher(const Regex::Regex &rx, const std::pair<std::string, std::string> &dir, bool filter);
+bool SongEntryMatcher(const Regex::Regex &rx, const MPD::MutableSong &s);
+
+template <bool Const>
+struct SongExtractor
+{
+	typedef SongExtractor type;
+
+	typedef typename NC::Menu<MPD::MutableSong>::Item MenuItem;
+	typedef typename std::conditional<Const, const MenuItem, MenuItem>::type Item;
+	typedef typename std::conditional<Const, const MPD::Song, MPD::Song>::type Song;
+
+	Song *operator()(Item &item) const
+	{
+		return &item.value();
+	}
+};
 
 }
 
+SongIterator TagsWindow::currentS()
+{
+	return makeSongIterator_<MPD::MutableSong>(current(), SongExtractor<false>());
+}
+
+ConstSongIterator TagsWindow::currentS() const
+{
+	return makeConstSongIterator_<MPD::MutableSong>(current(), SongExtractor<true>());
+}
+
+SongIterator TagsWindow::beginS()
+{
+	return makeSongIterator_<MPD::MutableSong>(begin(), SongExtractor<false>());
+}
+
+ConstSongIterator TagsWindow::beginS() const
+{
+	return makeConstSongIterator_<MPD::MutableSong>(begin(), SongExtractor<true>());
+}
+
+SongIterator TagsWindow::endS()
+{
+	return makeSongIterator_<MPD::MutableSong>(end(), SongExtractor<false>());
+}
+
+ConstSongIterator TagsWindow::endS() const
+{
+	return makeConstSongIterator_<MPD::MutableSong>(end(), SongExtractor<true>());
+}
+
+std::vector<MPD::Song> TagsWindow::getSelectedSongs()
+{
+	return {}; // TODO
+}
+
+/**********************************************************************/
+
 TagEditor::TagEditor() : FParser(0), FParserHelper(0), FParserLegend(0), FParserPreview(0), itsBrowsedDir("/")
 {
 	PatternsFile = Config.ncmpcpp_directory + "patterns.list";
 	SetDimensions(0, COLS);
 	
-	Dirs = new NC::Menu< std::pair<std::string, std::string> >(0, MainStartY, LeftColumnWidth, MainHeight, Config.titles_visibility ? "Directories" : "", Config.main_color, NC::Border::None);
+	Dirs = new NC::Menu< std::pair<std::string, std::string> >(0, MainStartY, LeftColumnWidth, MainHeight, Config.titles_visibility ? "Directories" : "", Config.main_color, NC::Border());
 	Dirs->setHighlightColor(Config.active_column_color);
 	Dirs->cyclicScrolling(Config.use_cyclic_scrolling);
 	Dirs->centeredCursor(Config.centered_cursor);
@@ -98,7 +154,7 @@ TagEditor::TagEditor() : FParser(0), FParserHelper(0), FParserLegend(0), FParser
 		menu << Charset::utf8ToLocale(menu.drawn()->value().first);
 	});
 	
-	TagTypes = new NC::Menu<std::string>(MiddleColumnStartX, MainStartY, MiddleColumnWidth, MainHeight, Config.titles_visibility ? "Tag types" : "", Config.main_color, NC::Border::None);
+	TagTypes = new NC::Menu<std::string>(MiddleColumnStartX, MainStartY, MiddleColumnWidth, MainHeight, Config.titles_visibility ? "Tag types" : "", Config.main_color, NC::Border());
 	TagTypes->setHighlightColor(Config.main_highlight_color);
 	TagTypes->cyclicScrolling(Config.use_cyclic_scrolling);
 	TagTypes->centeredCursor(Config.centered_cursor);
@@ -113,7 +169,7 @@ TagEditor::TagEditor() : FParser(0), FParserHelper(0), FParserLegend(0), FParser
 	TagTypes->addSeparator();
 	if (Config.titles_visibility)
 	{
-		TagTypes->addItem("Options", 1, 1);
+		TagTypes->addItem("Options", NC::List::Properties::Bold | NC::List::Properties::Inactive);
 		TagTypes->addSeparator();
 	}
 	TagTypes->addItem("Capitalize First Letters");
@@ -122,7 +178,7 @@ TagEditor::TagEditor() : FParser(0), FParserHelper(0), FParserLegend(0), FParser
 	TagTypes->addItem("Reset");
 	TagTypes->addItem("Save");
 	
-	Tags = new NC::Menu<MPD::MutableSong>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, Config.titles_visibility ? "Tags" : "", Config.main_color, NC::Border::None);
+	Tags = new TagsWindow(NC::Menu<MPD::MutableSong>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, Config.titles_visibility ? "Tags" : "", Config.main_color, NC::Border()));
 	Tags->setHighlightColor(Config.main_highlight_color);
 	Tags->cyclicScrolling(Config.use_cyclic_scrolling);
 	Tags->centeredCursor(Config.centered_cursor);
@@ -230,7 +286,7 @@ void TagEditor::refresh()
 
 void TagEditor::update()
 {
-	if (Dirs->reallyEmpty())
+	if (Dirs->empty())
 	{
 		Dirs->Window::clear();
 		Tags->clear();
@@ -239,22 +295,24 @@ void TagEditor::update()
 			Dirs->addItem(std::make_pair("..", getParentDirectory(itsBrowsedDir)));
 		else
 			Dirs->addItem(std::make_pair(".", "/"));
-		Mpd.GetDirectories(itsBrowsedDir, [this](std::string directory) {
-			Dirs->addItem(std::make_pair(getBasename(directory), directory));
-			if (directory == itsHighlightedDir)
+		MPD::DirectoryIterator directory = Mpd.GetDirectories(itsBrowsedDir), end;
+		for (; directory != end; ++directory)
+		{
+			Dirs->addItem(std::make_pair(getBasename(directory->path()), directory->path()));
+			if (directory->path() == itsHighlightedDir)
 				Dirs->highlight(Dirs->size()-1);
-		});
+		};
 		std::sort(Dirs->beginV()+1, Dirs->endV(),
 			LocaleBasedSorting(std::locale(), Config.ignore_leading_the));
 		Dirs->display();
 	}
 	
-	if (Tags->reallyEmpty())
+	if (Tags->empty())
 	{
 		Tags->reset();
-		Mpd.GetSongs(Dirs->current().value().second, [this](MPD::Song s) {
-			Tags->addItem(s);
-		});
+		MPD::SongIterator s = Mpd.GetSongs(Dirs->current()->value().second), end;
+		for (; s != end; ++s)
+			Tags->addItem(std::move(*s));
 		std::sort(Tags->beginV(), Tags->endV(),
 			LocaleBasedSorting(std::locale(), Config.ignore_leading_the));
 		Tags->refresh();
@@ -271,27 +329,208 @@ void TagEditor::update()
 	}
 }
 
-void TagEditor::enterPressed()
+bool TagEditor::enterDirectory()
 {
-	using Global::wFooter;
-	
-	if (w == Dirs)
+	bool result = false;
+	if (w == Dirs && !Dirs->empty())
 	{
-		bool has_subdirs = false;
-		Mpd.GetDirectories(Dirs->current().value().second, [&has_subdirs](std::string) {
-			has_subdirs = true;
-		});
+		MPD::DirectoryIterator directory = Mpd.GetDirectories(Dirs->current()->value().second), end;
+		bool has_subdirs = directory != end;
 		if (has_subdirs)
 		{
+			directory.finish();
 			itsHighlightedDir = itsBrowsedDir;
-			itsBrowsedDir = Dirs->current().value().second;
+			itsBrowsedDir = Dirs->current()->value().second;
 			Dirs->clear();
 			Dirs->reset();
+			result = true;
+		}
+	}
+	return result;
+}
+
+void TagEditor::mouseButtonPressed(MEVENT me)
+{
+	auto tryPreviousColumn = [this]() -> bool {
+		bool result = true;
+		if (w != Dirs)
+		{
+			if (previousColumnAvailable())
+				previousColumn();
+			else
+				result = false;
+		}
+		return result;
+	};
+	auto tryNextColumn = [this]() -> bool {
+		bool result = true;
+		if (w != Tags)
+		{
+			if (nextColumnAvailable())
+				nextColumn();
+			else
+				result = false;
+		}
+		return result;
+	};
+	if (w == FParserDialog)
+	{
+		if (FParserDialog->hasCoords(me.x, me.y))
+		{
+			if (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED))
+			{
+				FParserDialog->Goto(me.y);
+				if (me.bstate & BUTTON3_PRESSED)
+					runAction();
+			}
+			else
+				Screen<WindowType>::mouseButtonPressed(me);
+		}
+	}
+	else if (w == FParser || w == FParserHelper)
+	{
+		if (FParser->hasCoords(me.x, me.y))
+		{
+			if (w != FParser)
+			{
+				if (previousColumnAvailable())
+					previousColumn();
+				else
+					return;
+			}
+			if (size_t(me.y) < FParser->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
+			{
+				FParser->Goto(me.y);
+				if (me.bstate & BUTTON3_PRESSED)
+					runAction();
+			}
+			else
+				Screen<WindowType>::mouseButtonPressed(me);
+		}
+		else if (FParserHelper->hasCoords(me.x, me.y))
+		{
+			if (w != FParserHelper)
+			{
+				if (nextColumnAvailable())
+					nextColumn();
+				else
+					return;
+			}
+			scrollpadMouseButtonPressed(*FParserHelper, me);
+		}
+	}
+	else if (!Dirs->empty() && Dirs->hasCoords(me.x, me.y))
+	{
+		if (!tryPreviousColumn() || !tryPreviousColumn())
+			return;
+		if (size_t(me.y) < Dirs->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
+		{
+			Dirs->Goto(me.y);
+			if (me.bstate & BUTTON1_PRESSED)
+				enterDirectory();
 		}
 		else
-			Statusbar::print("No subdirectories found");
+			Screen<WindowType>::mouseButtonPressed(me);
+		Tags->clear();
 	}
-	else if (w == FParserDialog)
+	else if (!TagTypes->empty() && TagTypes->hasCoords(me.x, me.y))
+	{
+		if (w != TagTypes)
+		{
+			bool success;
+			if (w == Dirs)
+				success = tryNextColumn();
+			else
+				success = tryPreviousColumn();
+			if (!success)
+				return;
+		}
+		if (size_t(me.y) < TagTypes->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
+		{
+			if (!TagTypes->Goto(me.y))
+				return;
+			TagTypes->refresh();
+			Tags->refresh();
+			if (me.bstate & BUTTON3_PRESSED)
+				runAction();
+		}
+		else
+			Screen<WindowType>::mouseButtonPressed(me);
+	}
+	else if (!Tags->empty() && Tags->hasCoords(me.x, me.y))
+	{
+		if (!tryNextColumn() || !tryNextColumn())
+			return;
+		if (size_t(me.y) < Tags->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
+		{
+			Tags->Goto(me.y);
+			Tags->refresh();
+			if (me.bstate & BUTTON3_PRESSED)
+				runAction();
+		}
+		else
+			Screen<WindowType>::mouseButtonPressed(me);
+	}
+}
+
+/***********************************************************************/
+
+bool TagEditor::allowsSearching()
+{
+	return w == Dirs || w == Tags;
+}
+
+void TagEditor::setSearchConstraint(const std::string &constraint)
+{
+	if (w == Dirs)
+	{
+		m_directories_search_predicate = Regex::Filter<std::pair<std::string, std::string>>(
+			Regex::make(constraint, Config.regex_type),
+			std::bind(DirEntryMatcher, ph::_1, ph::_2, false)
+		);
+	}
+	else if (w == Tags)
+	{
+		m_songs_search_predicate = Regex::Filter<MPD::MutableSong>(
+			Regex::make(constraint, Config.regex_type),
+			SongEntryMatcher
+		);
+	}
+}
+
+void TagEditor::clearConstraint()
+{
+	if (w == Dirs)
+		m_directories_search_predicate.clear();
+	else if (w == Tags)
+		m_songs_search_predicate.clear();
+}
+
+bool TagEditor::find(SearchDirection direction, bool wrap, bool skip_current)
+{
+	bool result = false;
+	if (w == Dirs)
+		result = search(*Dirs, m_directories_search_predicate, direction, wrap, skip_current);
+	else if (w == Tags)
+		result = search(*Tags, m_songs_search_predicate, direction, wrap, skip_current);
+	return result;
+}
+
+/***********************************************************************/
+
+bool TagEditor::actionRunnable()
+{
+	// TODO: put something more refined here. It requires reworking
+	// runAction though, i.e. splitting it into smaller parts.
+	return (w == Tags && !Tags->empty())
+		|| w != Tags;
+}
+
+void TagEditor::runAction()
+{
+	using Global::wFooter;
+
+	if (w == FParserDialog)
 	{
 		size_t choice = FParserDialog->choice();
 		if (choice == 2) // cancel
@@ -301,9 +540,9 @@ void TagEditor::enterPressed()
 			return;
 		}
 		GetPatternList();
-		
+
 		// prepare additional windows
-		
+
 		FParserLegend->clear();
 		*FParserLegend << "%a - artist\n";
 		*FParserLegend << "%A - album artist\n";
@@ -320,7 +559,7 @@ void TagEditor::enterPressed()
 		for (auto it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
 			*FParserLegend << Config.color2 << " * " << NC::Color::End << (*it)->getName() << '\n';
 		FParserLegend->flush();
-		
+
 		if (!Patterns.empty())
 			Config.pattern = Patterns.front();
 		FParser->clear();
@@ -334,12 +573,12 @@ void TagEditor::enterPressed()
 		if (!Patterns.empty())
 		{
 			FParser->addSeparator();
-			FParser->addItem("Recent patterns", 1, 1);
+			FParser->addItem("Recent patterns", NC::List::Properties::Bold | NC::List::Properties::Inactive);
 			FParser->addSeparator();
 			for (std::list<std::string>::const_iterator it = Patterns.begin(); it != Patterns.end(); ++it)
 				FParser->addItem(*it);
 		}
-		
+
 		FParser->setTitle(choice == 0 ? "Get tags from filename" : "Rename files");
 		w = FParser;
 		FParserUsePreview = 1;
@@ -350,22 +589,21 @@ void TagEditor::enterPressed()
 	{
 		bool quit = 0;
 		size_t pos = FParser->choice();
-		
+
 		if (pos == 4) // save
 			FParserUsePreview = 0;
-		
+
 		if (pos == 0) // change pattern
 		{
-			Statusbar::lock();
-			Statusbar::put() << "Pattern: ";
-			std::string new_pattern = wFooter->getString(Config.pattern);
-			Statusbar::unlock();
-			if (!new_pattern.empty())
+			std::string new_pattern;
 			{
-				Config.pattern = new_pattern;
-				FParser->at(0).value() = "Pattern: ";
-				FParser->at(0).value() += Config.pattern;
+				Statusbar::ScopedLock slock;
+				Statusbar::put() << "Pattern: ";
+				new_pattern = wFooter->prompt(Config.pattern);
 			}
+			Config.pattern = new_pattern;
+			FParser->at(0).value() = "Pattern: ";
+			FParser->at(0).value() += Config.pattern;
 		}
 		else if (pos == 1 || pos == 4) // preview or proceed
 		{
@@ -435,10 +673,10 @@ void TagEditor::enterPressed()
 		}
 		else // list of patterns
 		{
-			Config.pattern = FParser->current().value();
+			Config.pattern = FParser->current()->value();
 			FParser->at(0).value() = "Pattern: " + Config.pattern;
 		}
-		
+
 		if (quit)
 		{
 			SavePatternList();
@@ -447,10 +685,10 @@ void TagEditor::enterPressed()
 			return;
 		}
 	}
-	
+
 	if ((w != TagTypes && w != Tags) || Tags->empty()) // after this point we start dealing with tags
 		return;
-	
+
 	EditedSongs.clear();
 	// if there are selected songs, perform operations only on them
 	if (hasSelected(Tags->begin(), Tags->end()))
@@ -464,50 +702,45 @@ void TagEditor::enterPressed()
 		for (auto it = Tags->begin(); it != Tags->end(); ++it)
 			EditedSongs.push_back(&it->value());
 	}
-	
+
 	size_t id = TagTypes->choice();
-	
+
 	if (w == TagTypes && id == 5)
 	{
-		bool yes = Actions::askYesNoQuestion("Number tracks?", Status::trace);
-		if (yes)
+		Actions::confirmAction("Number tracks?");
+		auto it = EditedSongs.begin();
+		for (unsigned i = 1; i <= EditedSongs.size(); ++i, ++it)
 		{
-			auto it = EditedSongs.begin();
-			for (unsigned i = 1; i <= EditedSongs.size(); ++i, ++it)
-			{
-				if (Config.tag_editor_extended_numeration)
-					(*it)->setTrack(boost::lexical_cast<std::string>(i) + "/" + boost::lexical_cast<std::string>(EditedSongs.size()));
-				else
-					(*it)->setTrack(boost::lexical_cast<std::string>(i));
-			}
-			Statusbar::print("Tracks numbered");
+			if (Config.tag_editor_extended_numeration)
+				(*it)->setTrack(boost::lexical_cast<std::string>(i) + "/" + boost::lexical_cast<std::string>(EditedSongs.size()));
+			else
+				(*it)->setTrack(boost::lexical_cast<std::string>(i));
+			// discard other track number tags
+			(*it)->setTrack("", 1);
 		}
-		else
-			Statusbar::print("Aborted");
+		Statusbar::print("Tracks numbered");
 		return;
 	}
-	
+
 	if (id < 11)
 	{
 		MPD::Song::GetFunction get = SongInfo::Tags[id].Get;
 		MPD::MutableSong::SetFunction set = SongInfo::Tags[id].Set;
 		if (id > 0 && w == TagTypes)
 		{
-			Statusbar::lock();
-			Statusbar::put() << NC::Format::Bold << TagTypes->current().value() << NC::Format::NoBold << ": ";
-			std::string new_tag = wFooter->getString(Tags->current().value().getTags(get, Config.tags_separator));
-			Statusbar::unlock();
+			Statusbar::ScopedLock slock;
+			Statusbar::put() << NC::Format::Bold << TagTypes->current()->value() << NC::Format::NoBold << ": ";
+			std::string new_tag = wFooter->prompt(Tags->current()->value().getTags(get));
 			for (auto it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
-				(*it)->setTags(set, new_tag, Config.tags_separator);
+				(*it)->setTags(set, new_tag);
 		}
 		else if (w == Tags)
 		{
-			Statusbar::lock();
-			Statusbar::put() << NC::Format::Bold << TagTypes->current().value() << NC::Format::NoBold << ": ";
-			std::string new_tag = wFooter->getString(Tags->current().value().getTags(get, Config.tags_separator));
-			Statusbar::unlock();
-			if (new_tag != Tags->current().value().getTags(get, Config.tags_separator))
-				Tags->current().value().setTags(set, new_tag, Config.tags_separator);
+			Statusbar::ScopedLock slock;
+			Statusbar::put() << NC::Format::Bold << TagTypes->current()->value() << NC::Format::NoBold << ": ";
+			std::string new_tag = wFooter->prompt(Tags->current()->value().getTags(get));
+			if (new_tag != Tags->current()->value().getTags(get))
+				Tags->current()->value().setTags(set, new_tag);
 			Tags->scroll(NC::Scroll::Down);
 		}
 	}
@@ -522,15 +755,14 @@ void TagEditor::enterPressed()
 			}
 			else if (w == Tags)
 			{
-				MPD::MutableSong &s = Tags->current().value();
+				Statusbar::ScopedLock slock;
+				MPD::MutableSong &s = Tags->current()->value();
 				std::string old_name = s.getNewName().empty() ? s.getName() : s.getNewName();
 				size_t last_dot = old_name.rfind(".");
 				std::string extension = old_name.substr(last_dot);
 				old_name = old_name.substr(0, last_dot);
-				Statusbar::lock();
 				Statusbar::put() << NC::Format::Bold << "New filename: " << NC::Format::NoBold;
-				std::string new_name = wFooter->getString(old_name);
-				Statusbar::unlock();
+				std::string new_name = wFooter->prompt(old_name);
 				if (!new_name.empty())
 					s.setNewName(new_name + extension);
 				Tags->scroll(NC::Scroll::Down);
@@ -587,278 +819,24 @@ void TagEditor::enterPressed()
 	}
 }
 
-void TagEditor::spacePressed()
-{
-	if (w == Tags && !Tags->empty())
-	{
-		Tags->current().setSelected(!Tags->current().isSelected());
-		w->scroll(NC::Scroll::Down);
-	}
-}
-
-void TagEditor::mouseButtonPressed(MEVENT me)
-{
-	auto tryPreviousColumn = [this]() -> bool {
-		bool result = true;
-		if (w != Dirs)
-		{
-			if (previousColumnAvailable())
-				previousColumn();
-			else
-				result = false;
-		}
-		return result;
-	};
-	auto tryNextColumn = [this]() -> bool {
-		bool result = true;
-		if (w != Tags)
-		{
-			if (nextColumnAvailable())
-				nextColumn();
-			else
-				result = false;
-		}
-		return result;
-	};
-	if (w == FParserDialog)
-	{
-		if (FParserDialog->hasCoords(me.x, me.y))
-		{
-			if (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED))
-			{
-				FParserDialog->Goto(me.y);
-				if (me.bstate & BUTTON3_PRESSED)
-					enterPressed();
-			}
-			else
-				Screen<WindowType>::mouseButtonPressed(me);
-		}
-	}
-	else if (w == FParser || w == FParserHelper)
-	{
-		if (FParser->hasCoords(me.x, me.y))
-		{
-			if (w != FParser)
-			{
-				if (previousColumnAvailable())
-					previousColumn();
-				else
-					return;
-			}
-			if (size_t(me.y) < FParser->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
-			{
-				FParser->Goto(me.y);
-				if (me.bstate & BUTTON3_PRESSED)
-					enterPressed();
-			}
-			else
-				Screen<WindowType>::mouseButtonPressed(me);
-		}
-		else if (FParserHelper->hasCoords(me.x, me.y))
-		{
-			if (w != FParserHelper)
-			{
-				if (nextColumnAvailable())
-					nextColumn();
-				else
-					return;
-			}
-			scrollpadMouseButtonPressed(*FParserHelper, me);
-		}
-	}
-	else if (!Dirs->empty() && Dirs->hasCoords(me.x, me.y))
-	{
-		if (!tryPreviousColumn() || !tryPreviousColumn())
-			return;
-		if (size_t(me.y) < Dirs->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
-		{
-			Dirs->Goto(me.y);
-			if (me.bstate & BUTTON1_PRESSED)
-				enterPressed();
-			else
-				spacePressed();
-		}
-		else
-			Screen<WindowType>::mouseButtonPressed(me);
-		Tags->clear();
-	}
-	else if (!TagTypes->empty() && TagTypes->hasCoords(me.x, me.y))
-	{
-		if (w != TagTypes)
-		{
-			bool success;
-			if (w == Dirs)
-				success = tryNextColumn();
-			else
-				success = tryPreviousColumn();
-			if (!success)
-				return;
-		}
-		if (size_t(me.y) < TagTypes->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
-		{
-			if (!TagTypes->Goto(me.y))
-				return;
-			TagTypes->refresh();
-			Tags->refresh();
-			if (me.bstate & BUTTON3_PRESSED)
-				enterPressed();
-		}
-		else
-			Screen<WindowType>::mouseButtonPressed(me);
-	}
-	else if (!Tags->empty() && Tags->hasCoords(me.x, me.y))
-	{
-		if (!tryNextColumn() || !tryNextColumn())
-			return;
-		if (size_t(me.y) < Tags->size() && (me.bstate & (BUTTON1_PRESSED | BUTTON3_PRESSED)))
-		{
-			Tags->Goto(me.y);
-			Tags->refresh();
-			if (me.bstate & BUTTON3_PRESSED)
-				enterPressed();
-		}
-		else
-			Screen<WindowType>::mouseButtonPressed(me);
-	}
-}
 
 /***********************************************************************/
 
-bool TagEditor::allowsFiltering()
-{
-	return w == Dirs || w == Tags;
-}
-
-std::string TagEditor::currentFilter()
-{
-	std::string filter;
-	if (w == Dirs)
-		filter = RegexFilter< std::pair<std::string, std::string> >::currentFilter(*Dirs);
-	else if (w == Tags)
-		filter = RegexFilter<MPD::MutableSong>::currentFilter(*Tags);
-	return filter;
-}
-
-void TagEditor::applyFilter(const std::string &filter)
-{
-	if (filter.empty())
-	{
-		if (w == Dirs)
-		{
-			Dirs->clearFilter();
-			Dirs->clearFilterResults();
-		}
-		else if (w == Tags)
-		{
-			Tags->clearFilter();
-			Tags->clearFilterResults();
-		}
-		return;
-	}
-	try
-	{
-		if (w == Dirs)
-		{
-			Dirs->showAll();
-			auto fun = boost::bind(DirEntryMatcher, _1, _2, true);
-			auto rx = RegexFilter< std::pair<std::string, std::string> >(
-				boost::regex(filter, Config.regex_type), fun);
-			Dirs->filter(Dirs->begin(), Dirs->end(), rx);
-		}
-		else if (w == Tags)
-		{
-			Tags->showAll();
-			auto rx = RegexFilter<MPD::MutableSong>(
-				boost::regex(filter, Config.regex_type), SongEntryMatcher);
-			Tags->filter(Tags->begin(), Tags->end(), rx);
-		}
-	}
-	catch (boost::bad_expression &) { }
-}
-
-/***********************************************************************/
-
-bool TagEditor::allowsSearching()
-{
-	return w == Dirs || w == Tags;
-}
-
-bool TagEditor::search(const std::string &constraint)
-{
-	if (constraint.empty())
-	{
-		if (w == Dirs)
-			Dirs->clearSearchResults();
-		else if (w == Tags)
-			Tags->clearSearchResults();
-		return false;
-	}
-	try
-	{
-		bool result = false;
-		if (w == Dirs)
-		{
-			auto fun = boost::bind(DirEntryMatcher, _1, _2, false);
-			auto rx = RegexFilter< std::pair<std::string, std::string> >(
-				boost::regex(constraint, Config.regex_type), fun);
-			result = Dirs->search(Dirs->begin(), Dirs->end(), rx);
-		}
-		else if (w == Tags)
-		{
-			auto rx = RegexFilter<MPD::MutableSong>(
-				boost::regex(constraint, Config.regex_type), SongEntryMatcher);
-			result = Tags->search(Tags->begin(), Tags->end(), rx);
-		}
-		return result;
-	}
-	catch (boost::bad_expression &)
-	{
-		return false;
-	}
-}
-
-void TagEditor::nextFound(bool wrap)
-{
-	if (w == Dirs)
-		Dirs->nextFound(wrap);
-	else if (w == Tags)
-		Tags->nextFound(wrap);
-}
-
-void TagEditor::prevFound(bool wrap)
-{
-	if (w == Dirs)
-		Dirs->prevFound(wrap);
-	else if (w == Tags)
-		Tags->prevFound(wrap);
-}
-
-/***********************************************************************/
-
-ProxySongList TagEditor::proxySongList()
-{
-	auto ptr = ProxySongList();
-	if (w == Tags)
-		ptr = ProxySongList(*Tags, [](NC::Menu<MPD::MutableSong>::Item &item) {
-			return &item.value();
-		});
-	return ptr;
-}
-
-bool TagEditor::allowsSelection()
-{
-	return w == Tags;
-}
-
-void TagEditor::reverseSelection()
+bool TagEditor::itemAvailable()
 {
 	if (w == Tags)
-		reverseSelectionHelper(Tags->begin(), Tags->end());
+		return !Tags->empty();
+	return false;
 }
 
-MPD::SongList TagEditor::getSelectedSongs()
+bool TagEditor::addItemToPlaylist(bool play)
 {
-	MPD::SongList result;
+	return addSongToPlaylist(*Tags->currentV(), play);
+}
+
+std::vector<MPD::Song> TagEditor::getSelectedSongs()
+{
+	std::vector<MPD::Song> result;
 	if (w == Tags)
 	{
 		for (auto it = Tags->begin(); it != Tags->end(); ++it)
@@ -866,7 +844,7 @@ MPD::SongList TagEditor::getSelectedSongs()
 				result.push_back(it->value());
 		// if no song was selected, add current one
 		if (result.empty() && !Tags->empty())
-			result.push_back(Tags->current().value());
+			result.push_back(Tags->current()->value());
 	}
 	return result;
 }
@@ -878,13 +856,14 @@ bool TagEditor::previousColumnAvailable()
 	bool result = false;
 	if (w == Tags)
 	{
-		if (!TagTypes->reallyEmpty() && !Dirs->reallyEmpty())
+		if (!TagTypes->empty() && !Dirs->empty())
 			result = true;
 	}
 	else if (w == TagTypes)
 	{
-		if (!Dirs->reallyEmpty())
-			result = ifAnyModifiedAskForDiscarding();
+		if (!Dirs->empty() && isAnyModified(*Tags))
+			Actions::confirmAction("There are pending changes, are you sure?");
+		result = true;
 	}
 	else if (w == FParserHelper)
 		result = true;
@@ -922,12 +901,12 @@ bool TagEditor::nextColumnAvailable()
 	bool result = false;
 	if (w == Dirs)
 	{
-		if (!TagTypes->reallyEmpty() && !Tags->reallyEmpty())
+		if (!TagTypes->empty() && !Tags->empty())
 			result = true;
 	}
 	else if (w == TagTypes)
 	{
-		if (!Tags->reallyEmpty())
+		if (!Tags->empty())
 			result = true;
 	}
 	else if (w == FParser)
@@ -944,7 +923,7 @@ void TagEditor::nextColumn()
 		w = TagTypes;
 		TagTypes->setHighlightColor(Config.active_column_color);
 	}
-	else if (w == TagTypes && TagTypes->choice() < 13 && !Tags->reallyEmpty())
+	else if (w == TagTypes && TagTypes->choice() < 13 && !Tags->empty())
 	{
 		TagTypes->setHighlightColor(Config.main_highlight_color);
 		w->refresh();
@@ -963,14 +942,6 @@ void TagEditor::nextColumn()
 
 /***********************************************************************/
 
-bool TagEditor::ifAnyModifiedAskForDiscarding()
-{
-	bool result = true;
-	if (isAnyModified(*Tags))
-		result = Actions::askYesNoQuestion("There are pending changes, are you sure?", Status::trace);
-	return result;
-}
-
 void TagEditor::LocateSong(const MPD::Song &s)
 {
 	if (myScreen == this)
@@ -1033,7 +1004,7 @@ void TagEditor::LocateSong(const MPD::Song &s)
 	}
 }
 
-namespace {//
+namespace {
 
 bool isAnyModified(const NC::Menu<MPD::MutableSong> &m)
 {
@@ -1136,7 +1107,7 @@ MPD::MutableSong::SetFunction IntoSetFunction(char c)
 
 std::string GenerateFilename(const MPD::MutableSong &s, const std::string &pattern)
 {
-	std::string result = s.toString(pattern, Config.tags_separator);
+	std::string result = Format::stringify<char>(Format::parse(pattern), &s);
 	removeInvalidCharsFromFilename(result, Config.generate_win32_compatible_filenames);
 	return result;
 }
@@ -1193,7 +1164,7 @@ std::string ParseFilename(MPD::MutableSong &s, std::string mask, bool preview)
 			{
 				MPD::MutableSong::SetFunction set = IntoSetFunction(it->first);
 				if (set)
-					s.setTags(set, it->second, Config.tags_separator);
+					s.setTags(set, it->second);
 			}
 			else
 				result << "%" << it->first << ": " << it->second << "\n";
@@ -1212,16 +1183,16 @@ std::string SongToString(const MPD::MutableSong &s)
 	return result.empty() ? Config.empty_tag : result;
 }
 
-bool DirEntryMatcher(const boost::regex &rx, const std::pair<std::string, std::string> &dir, bool filter)
+bool DirEntryMatcher(const Regex::Regex &rx, const std::pair<std::string, std::string> &dir, bool filter)
 {
 	if (dir.first == "." || dir.first == "..")
 		return filter;
-	return boost::regex_search(dir.first, rx);
+	return Regex::search(dir.first, rx);
 }
 
-bool SongEntryMatcher(const boost::regex &rx, const MPD::MutableSong &s)
+bool SongEntryMatcher(const Regex::Regex &rx, const MPD::MutableSong &s)
 {
-	return boost::regex_search(SongToString(s), rx);
+	return Regex::search(SongToString(s), rx);
 }
 
 }
