@@ -68,6 +68,11 @@ bool option_parser::run(std::istream &is, bool ignore_errors)
 			}
 		}
 	}
+	return true;
+}
+
+bool option_parser::initialize_undefined(bool ignore_errors)
+{
 	for (auto &p : m_parsers)
 	{
 		if (!p.second.defined())
@@ -75,7 +80,7 @@ bool option_parser::run(std::istream &is, bool ignore_errors)
 			try {
 				p.second.run_default();
 			} catch (std::exception &e) {
-				std::cerr << "Error while finalizing option \"" << p.first << "\": " << e.what() << "\n";
+				std::cerr << "Error while initializing option \"" << p.first << "\": " << e.what() << "\n";
 				if (ignore_errors)
 					return false;
 			}
