@@ -37,49 +37,55 @@
 struct Visualizer: Screen<NC::Window>, Tabbable
 {
 	Visualizer();
-	
+
 	virtual void switchTo() OVERRIDE;
 	virtual void resize() OVERRIDE;
-	
+
 	virtual std::wstring title() OVERRIDE;
 	virtual ScreenType type() OVERRIDE { return ScreenType::Visualizer; }
-	
+
 	virtual void update() OVERRIDE;
 	virtual void scroll(NC::Scroll) OVERRIDE { }
 
 	virtual int windowTimeout() OVERRIDE;
-	
-	virtual void enterPressed() OVERRIDE { }
-	virtual void spacePressed() OVERRIDE;
+
 	virtual void mouseButtonPressed(MEVENT) OVERRIDE { }
-	
+
+	virtual bool isLockable() OVERRIDE { return true; }
 	virtual bool isMergable() OVERRIDE { return true; }
-	
+
 	// private members
+	void ToggleVisualizationType();
 	void SetFD();
 	void ResetFD();
 	void FindOutputID();
-	
-protected:
-	virtual bool isLockable() OVERRIDE { return true; }
-	
+	void ResetAutoScaleMultiplier();
+
 private:
 	void DrawSoundWave(int16_t *, ssize_t, size_t, size_t);
+	void DrawSoundWaveStereo(int16_t *, int16_t *, ssize_t, size_t);
+	void DrawSoundWaveFill(int16_t *, ssize_t, size_t, size_t);
+	void DrawSoundWaveFillStereo(int16_t *, int16_t *, ssize_t, size_t);
+	void DrawSoundEllipse(int16_t *, ssize_t, size_t, size_t);
+	void DrawSoundEllipseStereo(int16_t *, int16_t *, ssize_t, size_t);
 #	ifdef HAVE_FFTW3_H
 	void DrawFrequencySpectrum(int16_t *, ssize_t, size_t, size_t);
+	void DrawFrequencySpectrumStereo(int16_t *, int16_t *, ssize_t, size_t);
 #	endif // HAVE_FFTW3_H
-	
+
 	int m_output_id;
 	boost::posix_time::ptime m_timer;
-	
+
 	int m_fifo;
-	unsigned m_samples;
+	size_t m_samples;
+	double m_auto_scale_multiplier;
 #	ifdef HAVE_FFTW3_H
-	unsigned m_fftw_results;
-	double *m_freq_magnitudes;
+	size_t m_fftw_results;
 	double *m_fftw_input;
 	fftw_complex *m_fftw_output;
 	fftw_plan m_fftw_plan;
+
+	std::vector<double> m_freq_magnitudes;
 #	endif // HAVE_FFTW3_H
 };
 
@@ -89,3 +95,4 @@ extern Visualizer *myVisualizer;
 
 #endif // NCMPCPP_VISUALIZER_H
 
+/* vim: set tabstop=4 softtabstop=4 shiftwidth=4 noexpandtab : */
